// WL_AGENT.C

#include "wl_def.h"
#pragma hdrstop

/*
=============================================================================

                                LOCAL CONSTANTS

=============================================================================
*/

#define MAXMOUSETURN    10


#define MOVESCALE       150l
#define BACKMOVESCALE   100l
#define ANGLESCALE      20

/*
=============================================================================

                                GLOBAL VARIABLES

=============================================================================
*/



//
// player state info
//
int32_t         thrustspeed;

word            plux,pluy;          // player coordinates scaled to unsigned

short           anglefrac;

objtype        *LastAttacker;

/*
=============================================================================

                                                 LOCAL VARIABLES

=============================================================================
*/


void    T_Player (objtype *ob);
void    T_Attack (objtype *ob);
void    T_Reload (objtype *ob);

statetype   s_player = {false,0,0,(statefunc) T_Player,NULL,NULL};
statetype   s_attack = {false,0,0,(statefunc) T_Attack,NULL,NULL};
statetype   s_reload = {false,0,0,(statefunc) T_Reload,NULL,NULL};

struct atkinf
{
    int8_t    tics,attack,frame;              // attack is 1 for gun, 2 for knife
} attackinfo[5][14] =
{
    { {6,0,1},{6,2,2},{6,0,3},{6,-1,4} },        // control gun rate of fire
    { {6,0,1},{24,1,2},{24,0,3},{24,-1,4} },        // third set second number 0 means one shoot for hit if button
    { {6,0,1},{6,1,2},{6,0,3},{6,-1,4} },        // 3 would mean one round as long as button held
    { {6,0,1},{6,1,2},{6,3,3},{6,-1,4} },        // 4 would mean two rounds as long as button held
    { {6,0,1},{6,1,2},{6,3,3},{6,-1,4} },
};

struct rloadinf
{
   char   tics,attack,frame;   //changed char to int8_t
} reloadinfo[NUMWEAPONS][14] =

// attack -1 goes back to first weapon frame
// attack 1 plays reloading sound
// attack 2 loads a full clip
// attack 3 loads 1 round at a time (could be used for weapons like shotguns)
// attack 4 goes back 2 frames (use with attack 3 for a looped "round by round" reloading animation)
{
{ {0,-1,0} }, // not used - knife doesn't reload
{ {20,1,0},{20,0,1},{20,0,2},{20,2,3},{20,-1,4} },
{ {20,1,0},{20,0,1},{20,0,2},{20,2,3},{20,-1,4} },
{ {20,1,0},{20,0,1},{20,0,2},{20,0,3},{20,0,4},{20,0,5},{20,0,6},{20,2,7},{20,-1,8} },
{ {20,1,0},{20,0,1},{20,0,2},{20,2,3},{20,-1,4} },
};

struct wepinfo
{
    short    weaponkey,ammotype,clipsize;
} weaponinfo[NUMWEAPONS] =
{
//
// weapon info - defines each weapon's selection key, the type of ammo it uses,
// and the maximum amount of ammo that can be stored in the clip. Weapons with
// a clipsize of "0" or an ammotype of "ammo_none" don't reload (ex: wp_knife)
// NOTE 9 is a special clip size only for mounted weapons
//
    {bt_readyknife,ammo_none,0},       // wp_knife
    {bt_readypistol,ammo_9mm,5},       // wp_pistol
    {bt_readymachinegun,ammo_30cal,8},      // wp_machinegun
    {bt_readychaingun,ammo_30cal,32}, // wp_chaingun
    {bt_readychaingun,ammo_emplaced,9},   // wp_emplacedgun
};

short MaxAmmo (int ammotype)
{
    short maxammo[NUMAMMOTYPES]=
    {
        100,    // ammo_9mm
        300,    // ammo_30cal
        100,    // ammo_emplaced
    };
    return maxammo[ammotype];
}

short ClipSize (int weapon) { return (weaponinfo[weapon].clipsize); }
boolean HasMaxAmmo (int ammotype) { if(gamestate.ammo[ammotype]>=MaxAmmo(ammotype)) return true; return false; }
boolean HasWeapon (int weapon) { if(gamestate.bestweapon&(1<<weapon)) return true; return false; }
void ReloadWeapon (int ammo);
void Cmd_Reload (void);
void Cmd_Heal (void);

#define CURRENTAMMO      (gamestate.ammo[weaponinfo[gamestate.weapon].ammotype])
#define CURRENTCLIP      (gamestate.clip[gamestate.weapon])


//===========================================================================

//----------

void Attack (void);
void Use (void);
void Search (objtype *ob);
void SelectWeapon (void);
void SelectItem (void);

//----------

boolean TryMove (objtype *ob);
void T_Player (objtype *ob);

void ClipMove (objtype *ob, int32_t xmove, int32_t ymove);

/*
=============================================================================

                                CONTROL STUFF

=============================================================================
*/

/*
======================
=
= CheckWeaponChange
=
= Keys 1-4 change weapons
=
======================
*/

boolean WeaponAvailable (int weapon)
{
    if (!HasWeapon(weapon))
        return false;

    if (weaponinfo[weapon].ammotype == ammo_none)
        return true;

    if (gamestate.ammo[weaponinfo[weapon].ammotype] || gamestate.clip[weapon])
        return true;

    return false;
}

boolean ReadyToFire (int weapon)
{
    if (!WeaponAvailable(weapon))
        return false;

    if (ClipSize(weapon) > 0)
        return (gamestate.clip[weapon] > 0);

    return true;
}

weapontype BestAvailableWeapon (void)
{
    int bestweapon = 0;

    for (int i = 0; i < NUMWEAPONS; i++)
    {
        if(WeaponAvailable(i))
            bestweapon = i;
    }

    return (weapontype)bestweapon;
}

weapontype CheckWeaponChoice (short weaponkey)
{
    weapontype firstchoice = (weapontype)-1;
    weapontype lastchosen = gamestate.lastchosenweapon[weaponkey];

    if (weaponkey != weaponinfo[gamestate.weapon].weaponkey && lastchosen > 0
    && WeaponAvailable(lastchosen))
        return lastchosen;

    for (int i = 0; i < NUMWEAPONS; i++)
    {
        if (!WeaponAvailable(i) || weaponinfo[i].weaponkey != weaponkey)
            continue;

        if (firstchoice == -1) firstchoice = (weapontype)i;

        if (weaponkey == weaponinfo[gamestate.weapon].weaponkey && i > gamestate.weapon)
            return (weapontype)i;
    }

    return firstchoice;
}

void CheckWeaponChange (void)
{
    int newWeapon = -1;
    weapontype bestweapon = BestAvailableWeapon();

    if (gamestate.reloading)    // can't switch weapons while reloading
        return;
        if (gamestate.weapon != wp_emplacedgun){//mounted start end
           gamestate.clip[4] = 0;
           gamestate.ammo[ammo_emplaced] = 0;}//mounted start end

#ifdef _arch_dreamcast //dreamcast controls
    int joyx, joyy;
    IN_GetJoyFineDelta (&joyx, &joyy);
    if(joyx < -64)
        buttonstate[bt_prevweapon] = true;
    if(joyx > 64)
        buttonstate[bt_nextweapon] = true;
    if(joyy < -64)
        buttonstate[bt_healthkit] = true;
    else if(joyy > 64)
        buttonstate[bt_reload] = true;

#endif

    if(buttonstate[bt_nextweapon] && !buttonheld[bt_nextweapon])
    {
        if (gamestate.weapon+1 > bestweapon) newWeapon = 0;
        else
        {
            for(int i = gamestate.weapon+1; i <= bestweapon; i++)
            {
                if(WeaponAvailable(i))
                {
                    newWeapon = i;
                    break;
                }
            }
        }
    }
    else if(buttonstate[bt_prevweapon] && !buttonheld[bt_prevweapon])
    {
        if(gamestate.weapon-1 < 0) newWeapon = bestweapon;
        else
        {
            for(int i = gamestate.weapon-1; i >= 0; i--)
            {
                if(WeaponAvailable(i))
                {
                    newWeapon = i;
                    break;
                }
            }
        }
    }
    else
    {
        for(int i = bt_readyknife; i < NUMBUTTONS; i++)
        {
            if (buttonstate[i] && !buttonheld[i])
            {
                newWeapon = CheckWeaponChoice(i);
                break;
            }
        }
    }

    if(newWeapon != -1)
    {
        if (!gamestate.clip[newWeapon])
            Cmd_Reload ();
        gamestate.weapon = gamestate.chosenweapon =
        gamestate.lastchosenweapon[weaponinfo[newWeapon].weaponkey] = (weapontype) newWeapon;
        //DrawWeapon();
        //DrawAmmo();
    }
}

/*
=======================
=
= ControlMovement
=
= Takes controlx,controly, and buttonstate[bt_strafe]
=
= Changes the player's angle and position
=
= There is an angle hack because when going 70 fps, the roundoff becomes
= significant
=
=======================
*/

void ControlMovement (objtype *ob)
{
    int32_t oldx,oldy;
    int     angle;
    int     angleunits;

    thrustspeed = 0;

    oldx = player->x;
    oldy = player->y;


/*    //start for strafe
    if ( buttonstate[bt_strafeA] && !buttonheld[bt_strafeA])
   {
   //
   // strafing
   //
   //
      if (strafe > 0)
      {
         angle = ob->angle - ANGLES/4;
         if (angle < 0)
            angle += ANGLES;
         Thrust (angle,strafe*MOVESCALE);   // move to left
      }
      else if (strafe < 0)
      {
         angle = ob->angle + ANGLES/4;
         if (angle >= ANGLES)
            angle -= ANGLES;
         Thrust (angle,-strafe*MOVESCALE);   // move to right
      }
   }

    if ( buttonstate[bt_strafeD] && !buttonheld[bt_strafeD])
   {
   //
   // strafing
   //
   //
      if (strafe > 0)
      {
         angle = ob->angle - ANGLES/4;
         if (angle < 0)
            angle += ANGLES;
         Thrust (angle,strafe*MOVESCALE);   // move to left
      }
      else if (strafe < 0)
      {
         angle = ob->angle + ANGLES/4;
         if (angle >= ANGLES)
            angle -= ANGLES;
         Thrust (angle,-strafe*MOVESCALE);   // move to right
      }
   }
*/
    //end for strafe


    //MOUNTED START
         //
        // If mounted machinegun is selected then...
        //

        if (gamestate.weapon == wp_emplacedgun)
        {
           if(buttonstate[bt_use] && !buttonheld[bt_use]){
             buttonheld[bt_use] = true;
             gamestate.weapon = gamestate.chosenweapon = gamestate.oldweapon;
           }
        }
        //MOUNTED END



    if(buttonstate[bt_strafeleft])
    {
        angle = ob->angle + ANGLES/4;
        if(angle >= ANGLES)
            angle -= ANGLES;
        if(buttonstate[bt_run])
            Thrust(angle, RUNMOVE * MOVESCALE * tics);
        else
            Thrust(angle, BASEMOVE * MOVESCALE * tics);
    }

    if(buttonstate[bt_straferight])
    {
        angle = ob->angle - ANGLES/4;
        if(angle < 0)
            angle += ANGLES;
        if(buttonstate[bt_run])
            Thrust(angle, RUNMOVE * MOVESCALE * tics );
        else
            Thrust(angle, BASEMOVE * MOVESCALE * tics);
    }
    //
    // side to side move
    //
    if (buttonstate[bt_strafe])
    {
        //
        // strafing
        //
        //
        //MOUNTED START
                  if(gamestate.weapon < wp_emplacedgun)
            {
//MOUNTED END

        if (controlx > 0)
        {
            angle = ob->angle - ANGLES/4;
            if (angle < 0)
                angle += ANGLES;
            Thrust (angle,controlx*MOVESCALE);      // move to left
        }
        else if (controlx < 0)
        {
            angle = ob->angle + ANGLES/4;
            if (angle >= ANGLES)
                angle -= ANGLES;
            Thrust (angle,-controlx*MOVESCALE);     // move to right
        }
    }
} //mounted end start


    else
    {
        //
        // not strafing
        //
        anglefrac += controlx;
        angleunits = anglefrac/ANGLESCALE;
        anglefrac -= angleunits*ANGLESCALE;
        ob->angle -= angleunits;

        if (ob->angle >= ANGLES)
            ob->angle -= ANGLES;
        if (ob->angle < 0)
            ob->angle += ANGLES;

    }
    //MOUNTED START
          if(gamestate.chosenweapon == wp_emplacedgun) return;
//MOUNTED END

    //
    // forward/backwards move
    //
    if (controly < 0)
    {
        Thrust (ob->angle,-controly*MOVESCALE); // move forwards
    }
    else if (controly > 0)
    {
        angle = ob->angle + ANGLES/2;
        if (angle >= ANGLES)
            angle -= ANGLES;
        Thrust (angle,controly*BACKMOVESCALE);          // move backwards
    }

    if (gamestate.victoryflag)              // watching the BJ actor
        return;
}

/*
=============================================================================

                            STATUS WINDOW STUFF

=============================================================================
*/


/*
==================
=
= StatusDrawPic
=
==================
*/

void StatusDrawPic (unsigned x, unsigned y, unsigned picnum)
{
    //LatchDrawPicScaledCoord ((screenWidth-scaleFactor*320)/16 + scaleFactor*x,
    //    screenHeight-scaleFactor*(STATUSLINES-y),picnum);
}

void StatusDrawFace(unsigned picnum)
{
    //StatusDrawPic(17, 4, picnum);
    //StatusDrawLCD(picnum);
}


/*
==================
=
= DrawFace
=
==================
*/

void DrawFace (void)
{
    if(viewsize == 21 && ingame) return;
    if (SD_SoundPlaying() == GETGATLINGSND)
        StatusDrawFace(GOTGATLINGPIC);
    else if (gamestate.health)
    {
#ifdef SPEAR
        if (godmode)
            StatusDrawFace(GODMODEFACE1PIC+gamestate.faceframe);
        else
#endif
            StatusDrawFace(FACE1APIC+3*((100-gamestate.health)/16)+gamestate.faceframe);
    }
    else
    {
#ifndef SPEAR
        if (LastAttacker && LastAttacker->obclass == needleobj)
            StatusDrawFace(MUTANTBJPIC);
        else
#endif
            StatusDrawFace(FACE8APIC);
    }
}

/*
===============
=
= UpdateFace
=
= Calls draw face if time to change
=
===============
*/

int facecount = 0;
int facetimes = 0;

void UpdateFace (void)
{
    // don't make demo depend on sound playback
    if(demoplayback || demorecord)
    {
        if(facetimes > 0)
        {
            facetimes--;
            return;
        }
    }
    else if(SD_SoundPlaying() == GETGATLINGSND)
        return;

    facecount += tics;
    if (facecount > US_RndT())
    {
        gamestate.faceframe = (US_RndT()>>6);
        if (gamestate.faceframe==3)
            gamestate.faceframe = 1;

        facecount = 0;
        DrawFace ();
    }
}



/*
===============
=
= LatchNumber
=
= right justifies and pads with blanks
=
===============
*/

static void LatchNumber (int x, int y, unsigned width, int32_t number)
{
    unsigned length,c;
    char    str[20];

    ltoa (number,str,10);

    length = (unsigned) strlen (str);

    while (length<width)
    {
        StatusDrawPic (x,y,N_BLANKPIC);
        x++;
        width--;
    }

    c = length <= width ? 0 : length-width;

    while (c<length)
    {
        StatusDrawPic (x,y,str[c]-'0'+ N_0PIC);
        x++;
        c++;
    }
}


/*
===============
=
= DrawHealth
=
===============
*/

void DrawHealth (void)
{
//   if(viewsize == 21 && ingame) return;
//    LatchNumber (21,16,3,gamestate.health);

}





//void DrawHealth (void)
//{
//    if(viewsize == 21 && ingame) return;
//    LatchNumber (21,16,3,gamestate.health);
//}

/*
===============
=
= TakeDamage
=
===============
*/

void TakeDamage (int points,objtype *attacker)
{
    LastAttacker = attacker;

    if (gamestate.victoryflag)
        return;
    if (gamestate.difficulty==gd_baby)
        points>>=2;

    if (!godmode)
        gamestate.health -= points;

    if (gamestate.health<=0)
    {
        gamestate.health = 0;
        playstate = ex_died;
        killerobj = attacker;
    }


    if (godmode != 2)
        StartDamageFlash (points);

    DrawHealth ();
    DrawFace ();

    //
    // MAKE BJ'S EYES BUG IF MAJOR DAMAGE!
    //
#ifdef SPEAR
    if (points > 30 && gamestate.health!=0 && !godmode && viewsize != 21)
    {
        StatusDrawFace(BJOUCHPIC);
        facecount = 0;
    }
#endif

   // BEGIN ZOOM BLOCK
   StartDamageShock( points );
   // END ZOOM BLOCK

   /*if (gamestate.health < 21)//auto medkit does not work totaly right some times gets skipped over
   {
       if (gamestate.medical > 0)
       {
       SD_PlaySound (PERCENT100SND);
       gamestate.medical--;

       if (gamestate.difficulty == gd_baby) HealSelf (40);
       if (gamestate.difficulty == gd_easy) HealSelf (30);
       if (gamestate.difficulty == gd_medium) HealSelf (25);
       if (gamestate.difficulty == gd_hard) HealSelf (20);
       }
   }*/
}

/*
===============
=
= HealSelf
=
===============
*/

void HealSelf (int points)
{
    gamestate.health += points;
    if (gamestate.health>100)
        gamestate.health = 100;

    DrawHealth ();
    DrawFace ();
}


//===========================================================================


/*
===============
=
= DrawLevel
=
===============
*/

void DrawLevel (void)
{
    if(viewsize == 21 && ingame) return;
#ifdef SPEAR
    if (gamestate.mapon == 20)
        LatchNumber (2,16,2,18);
    else
#endif
        LatchNumber (2,16,2,gamestate.mapon+1);
}

//===========================================================================


/*
===============
=
= DrawLives
=
===============
*/

void DrawLives (void)
{
    if(viewsize == 21 && ingame) return;
    LatchNumber (14,16,1,gamestate.lives);
}


/*
===============
=
= GiveExtraMan
=
===============
*/

void GiveExtraMan (void)
{
    if (gamestate.lives<9)
        gamestate.lives++;
    DrawLives ();
    SD_PlaySound (BONUS1UPSND);
}

//===========================================================================

/*
===============
=
= DrawScore
=
===============
*/

void DrawScore (void)
{
    if(viewsize == 21 && ingame) return;
    LatchNumber (6,16,6,gamestate.score);
}

/*
===============
=
= GivePoints
=
===============
*/

void GivePoints (int32_t points)
{
    gamestate.score += points;
    while (gamestate.score >= gamestate.nextextra)
    {
        gamestate.nextextra += EXTRAPOINTS;
        //GiveExtraMan ();
    }
    DrawScore ();
}

//===========================================================================

/*
==================
=
= DrawWeapon
=
==================
*/

void DrawWeapon (void)
{
    if(viewsize == 21 && ingame) return;
    //StatusDrawPic (32,8,KNIFEPIC+gamestate.weapon);
    if (!ClipSize(gamestate.weapon))
    {   // if weapon has no clip, draw 3 blank spaces instead
        StatusDrawPic (26,16,N_BLANKPIC);
        StatusDrawPic (27,16,N_BLANKPIC);
        StatusDrawPic (28,16,N_BLANKPIC);
    }
    else // draw ammo in clip
        LatchNumber (26,16,3,CURRENTCLIP);
}

/*
==================
=
= DrawKeys
=
==================
*/

void DrawKeys (void)
{
/*
    if(viewsize == 21 && ingame) return;
        StatusDrawPic (30,4,GOLDKEYPIC);
    if (gamestate.keys & 1)
   else
        StatusDrawPic (30,4,NOKEYPIC);

    if (gamestate.keys & 2)
        StatusDrawPic (30,20,SILVERKEYPIC);
    else
        StatusDrawPic (30,20,NOKEYPIC);
*/
}

/*
==================
=
= GiveWeapon
=
==================
*/

void GiveWeapon (int weapon)
{
    if (!HasWeapon(weapon))
    {
        if (weapon > BestAvailableWeapon())
        {
            gamestate.chosenweapon = (weapontype)weapon;
            if (!gamestate.attackframe && !gamestate.reloading)
                gamestate.weapon = gamestate.chosenweapon;
        }
        gamestate.bestweapon = (weapontype)(gamestate.bestweapon | (1<<weapon));
    }

    if (ClipSize(weapon) > 0 && !gamestate.clip[weapon])
        gamestate.clip[weapon] = ClipSize(weapon);
    else
        GiveAmmo (ClipSize(weapon),weaponinfo[weapon].ammotype);

    DrawAmmo ();
    DrawWeapon ();
}
//===========================================================================

/*
===============
=
= DrawAmmo
=
===============
*/

void DrawAmmo (void)
{
    if(viewsize == 21 && ingame) return;
    if (!ClipSize(gamestate.weapon))
    {   // if weapon doesn't use ammo, draw 3 blank spaces instead
        StatusDrawPic (32,16,N_BLANKPIC);
        StatusDrawPic (33,16,N_BLANKPIC);
        StatusDrawPic (34,16,N_BLANKPIC);
    }
    else // draw ammo in reserve
        LatchNumber (32,16,3,CURRENTAMMO);
}


/*
===============
=
= GiveAmmo
=
===============
*/
void GiveAmmo (int ammo, int ammotype)
{
    if (ammotype == ammo_none || ammotype >= NUMAMMOTYPES)
        return;

    gamestate.ammo[ammotype] += ammo;
    if (gamestate.ammo[ammotype] > MaxAmmo(ammotype))
        gamestate.ammo[ammotype] = MaxAmmo(ammotype);
    if (gamestate.weapon!=gamestate.chosenweapon && WeaponAvailable(gamestate.chosenweapon)
    && !gamestate.attackframe && !gamestate.reloading)
    {
        gamestate.weapon = gamestate.chosenweapon;
        DrawWeapon ();
        if (!ReadyToFire(gamestate.weapon))
            Cmd_Reload ();
    }

    DrawAmmo ();
}
//===========================================================================

/*
==================
=
= GiveKey
=
==================
*/

void GiveKey (int key)
{
     int p;
     int t;


    gamestate.keys |= (1<<key);
    DrawKeys ();

    t = key + 1;
    for (p = 0; p < MAXDOORS; p++)
        if (doorobjlist[p].lock == t)
         if (MAPSPOT(doorobjlist[p].tilex, doorobjlist[p].tiley, 1) == ENEMYBLOCK)
            MAPSPOT(doorobjlist[p].tilex, doorobjlist[p].tiley, 1) = 0;
}



/*
=============================================================================

                                MOVEMENT

=============================================================================
*/


/*
===================
=
= GetBonus
=
===================
*/
void GetBonus (statobj_t *check)
{
    switch (check->itemnumber)
    {
        case    bo_firstaid:
                if (gamestate.medical == 9)
                   return;

                SD_PlaySound (HEALTH2SND);
                   gamestate.medical++;
                   break;
//            if (gamestate.health == 100)
//                return;
//
//            SD_PlaySound (HEALTH2SND);
//            HealSelf (25);
//            break;

        case    bo_key1:                  //DCupdated
                gamestate.exseqtrigger = true;//trigger event
            break;                         //DCUpdated
        case    bo_key2:
            GiveKey (check->itemnumber - bo_key1);
            SD_PlaySound (GETKEYSND);
            break;
        case    bo_key3:
            GiveKey (check->itemnumber - bo_key1);
            SD_PlaySound (GETKEYSND);
            break;
        case    bo_key4:
            //GiveKey (check->itemnumber - bo_key1);
            GiveKey (check->itemnumber - bo_key1);
            SD_PlaySound (GETKEYSND);
            break;

        case    bo_cross:
            SD_PlaySound (BONUS1SND);
            GivePoints (0);
            gamestate.treasurecount++;
            break;
        case    bo_chalice:
            SD_PlaySound (BONUS2SND);
            GivePoints (0);
            gamestate.treasurecount++;
            break;
        case    bo_bible:
            SD_PlaySound (BONUS3SND);
            GivePoints (0);
            gamestate.treasurecount++;
            break;
        case    bo_crown:
            SD_PlaySound (BONUS4SND);
            GivePoints (0);
            gamestate.treasurecount++;
            break;

        case    bo_clip:
            if (HasMaxAmmo(ammo_9mm))
                return;

            SD_PlaySound (GETAMMOSND);
            GiveAmmo (8,ammo_9mm);
            break;
        case    bo_clip2:
            if (HasMaxAmmo(ammo_9mm))
                return;

            SD_PlaySound (GETAMMOSND);
            GiveAmmo (4,ammo_9mm);
            break;

#ifdef SPEAR
        case    bo_25clip:
            if (HasMaxAmmo(ammo_30cal))
                return;

            SD_PlaySound (GETAMMOBOXSND);
            GiveAmmo (25,ammo_30cal);
            break;
#endif

        case    bo_machinegun:
            if (HasMaxAmmo(weaponinfo[wp_machinegun].ammotype) && HasWeapon(wp_machinegun))
                return;
            SD_PlaySound (GETAMMOSND);
            GiveWeapon (wp_machinegun);
            break;
        case    bo_chaingun:
            if (HasMaxAmmo(weaponinfo[wp_chaingun].ammotype) && HasWeapon(wp_chaingun))
                return;
            SD_PlaySound (GETAMMOSND);
            facetimes = 38;
            GiveWeapon (wp_chaingun);

            if(viewsize != 21)
                StatusDrawFace (GOTGATLINGPIC);
            facecount = 0;
            break;

            //MOUNTED START
       case    bo_emplacedgun:
                if (gamestate.weapon < wp_emplacedgun)
                {
                   if (buttonstate[bt_use] && !buttonheld[bt_use])
                   {
            GiveWeapon (wp_emplacedgun);
                        ResetZoomEffects();
                        gamestate.ironsights = false;
                        gamestate.clip[4] = 1;
                        gamestate.ammo[ammo_emplaced] = 10;
                      buttonheld[bt_use] = true;
                      gamestate.oldweapon = gamestate.weapon;
                      gamestate.weapon = gamestate.chosenweapon = wp_emplacedgun;
                   }
                }
                //return;
                break;
                //MOUNTED END



        //
        // QUICKSAVE for checkpoint
        //
        case bo_checkpoint:
             {
        //    CP_SaveGame (1);
        break;
}

            //static int SaveGamesAvail[10];
            //LSItems.curpos = { LSM_X, LSM_Y, lengthof(LSMenu), 0, 24 };
            //CP_SaveGame(1);
            //if (SaveGamesAvail[(LSM_X, LSM_Y, lengthof(LSMenu), 0, 24)] && pickquick)
       /*     {
                CA_CacheGrChunk (STARTFONT + 1);
                fontnumber = 1;
                Message (STR_SAVING "...");
                CP_SaveGame (1);
                fontnumber = 0;
            } */
            //else
            //return;
       /*     {
                CacheLump (BACKDROP_LUMP_START, BACKDROP_LUMP_END);
                CA_CacheGrChunk (C_CURSOR1PIC);

                VW_FadeOut ();
                if(screenHeight % 200 != 0)
                    VL_ClearScreen(0);

                lastgamemusicoffset = StartCPMusic (MENUSONG);
                pickquick = CP_SaveGame (0);

                SETFONTCOLOR (0, 15);
                IN_ClearKeysDown ();
                VW_FadeOut();
                if(viewsize != 21)
                    DrawPlayScreen ();

                if (!startgame && !loadedgame)
                    ContinueMusic (lastgamemusicoffset);

                if (loadedgame)
                    playstate = ex_abort;
                lasttimecount = GetTimeCount ();
                 frameon = 0;

                if (MousePresent && IN_IsInputGrabbed())
                    IN_CenterMouse();     // Clear accumulated mouse movement

                UnCacheLump (BACKDROP_LUMP_START, BACKDROP_LUMP_END);
            }
            return 1; */



        case    bo_fullheal:
            SD_PlaySound (BONUS1UPSND);
            HealSelf (99);
            GiveAmmo (25,ammo_9mm);
            //GiveExtraMan ();
            gamestate.treasurecount++;
            break;

        case    bo_food:
            if (gamestate.health == 100)
                return;

            SD_PlaySound (HEALTH1SND);
            HealSelf (10);
            break;

        case    bo_alpo:
            if (gamestate.health == 100)
                return;

            SD_PlaySound (HEALTH1SND);
            HealSelf (4);
            break;

        case    bo_gibs:
            if (gamestate.health >10)
                return;

            SD_PlaySound (SLURPIESND);
            HealSelf (1);
            break;

#ifdef SPEAR
        case    bo_spear:
            spearflag = true;
            spearx = player->x;
            speary = player->y;
            spearangle = player->angle;
            playstate = ex_completed;
#endif
    }

    StartBonusFlash ();
    check->shapenum = -1;                   // remove from list
}

/*
===================
=
= TryMove
=
= returns true if move ok
= debug: use pointers to optimize
===================
*/

boolean TryMove (objtype *ob)
{
    int         xl,yl,xh,yh,x,y;
    objtype    *check;
    int32_t     deltax,deltay;

    xl = (ob->x-PLAYERSIZE) >>TILESHIFT;
    yl = (ob->y-PLAYERSIZE) >>TILESHIFT;

    xh = (ob->x+PLAYERSIZE) >>TILESHIFT;
    yh = (ob->y+PLAYERSIZE) >>TILESHIFT;

#define PUSHWALLMINDIST PLAYERSIZE

    //
    // check for solid walls
    //
    for (y=yl;y<=yh;y++)
    {
        for (x=xl;x<=xh;x++)
        {
            check = actorat[x][y];
            if (check && !ISPOINTER(check))
            {
                if(tilemap[x][y]==64 && x==pwallx && y==pwally)   // back of moving pushwall?
                {
                    switch(pwalldir)
                    {
                        case di_north:
                            if(ob->y-PUSHWALLMINDIST<=(pwally<<TILESHIFT)+((63-pwallpos)<<10))
                                return false;
                            break;
                        case di_west:
                            if(ob->x-PUSHWALLMINDIST<=(pwallx<<TILESHIFT)+((63-pwallpos)<<10))
                                return false;
                            break;
                        case di_east:
                            if(ob->x+PUSHWALLMINDIST>=(pwallx<<TILESHIFT)+(pwallpos<<10))
                                return false;
                            break;
                        case di_south:
                            if(ob->y+PUSHWALLMINDIST>=(pwally<<TILESHIFT)+(pwallpos<<10))
                                return false;
                            break;
                    }
                }
                else return false;
            }
        }
    }

    //
    // check for actors
    //
    if (yl>0)
        yl--;
    if (yh<MAPSIZE-1)
        yh++;
    if (xl>0)
        xl--;
    if (xh<MAPSIZE-1)
        xh++;

    for (y=yl;y<=yh;y++)
    {
        for (x=xl;x<=xh;x++)
        {
            check = actorat[x][y];
            if (ISPOINTER(check) && check != player && (check->flags & FL_SHOOTABLE) )
            {
                deltax = ob->x - check->x;
                if (deltax < -MINACTORDIST || deltax > MINACTORDIST)
                    continue;
                deltay = ob->y - check->y;
                if (deltay < -MINACTORDIST || deltay > MINACTORDIST)
                    continue;

                return false;
            }
        }
    }

    return true;
}


/*
===================
=
= ClipMove
=
===================
*/

void ClipMove (objtype *ob, int32_t xmove, int32_t ymove)
{
    int32_t    basex,basey;

    basex = ob->x;
    basey = ob->y;

    ob->x = basex+xmove;
    ob->y = basey+ymove;
    if (TryMove (ob))
        return;

#ifndef REMDEBUG
    if (noclip && ob->x > 2*TILEGLOBAL && ob->y > 2*TILEGLOBAL
        && ob->x < (((int32_t)(mapwidth-1))<<TILESHIFT)
        && ob->y < (((int32_t)(mapheight-1))<<TILESHIFT) )
        return;         // walk through walls
#endif

//    if (!SD_SoundPlaying())
//        SD_PlaySound (HITWALLSND);

    ob->x = basex+xmove;
    ob->y = basey;
    if (TryMove (ob))
        return;

    ob->x = basex;
    ob->y = basey+ymove;
    if (TryMove (ob))
        return;

    ob->x = basex;
    ob->y = basey;
}

//==========================================================================

/*
===================
=
= VictoryTile
=
===================
*/

void VictoryTile (void)
{
#ifndef SPEAR
    //SpawnBJVictory ();
    playstate = ex_victorious;                              // exit castle tile

#endif

    gamestate.victoryflag = true;
}



/*
===================
=
= Thrust
=
===================
*/
// For player movement in demos exactly as in the original Wolf3D v1.4 source code
static fixed FixedByFracOrig(fixed a, fixed b)
{
    int sign = 0;
    if(b == 65536) b = 65535;
    else if(b == -65536) b = 65535, sign = 1;
    else if(b < 0) b = (-b), sign = 1;

    if(a < 0)
    {
        a = -a;
        sign = !sign;
    }
    fixed res = (fixed)(((int64_t) a * b) >> 16);
    if(sign)
        res = -res;
    return res;
}

void Thrust (int angle, int32_t speed)
{
    int32_t xmove,ymove;
    unsigned offset;

 if (gamestate.bobdir != 2)
  {gamestate.bobber++;}
  if (gamestate.bobdir == 2)
  {gamestate.bobber--;}
  if (gamestate.bobber == 10)
  {gamestate.bobdir = 2;}
  if (gamestate.bobber == 0)
  {gamestate.bobdir = 1;}
  if (gamestate.bobdir2 != 2)
  {gamestate.bobber2++;}
  if (gamestate.bobdir2 == 2)
  {gamestate.bobber2--;}
  if (gamestate.bobber2 == 20)
  {gamestate.bobdir2 = 2;}
  if (gamestate.bobber2 == 0)
  {gamestate.bobdir2 = 1;}

    //
    // ZERO FUNNY COUNTER IF MOVED!
    //
#ifdef SPEAR
    if (speed)
        funnyticount = 0;
#endif

    thrustspeed += speed;
    //
    // moving bounds speed
    //
    if (speed >= MINDIST*2)
        speed = MINDIST*2-1;

    xmove = DEMOCHOOSE_ORIG_SDL(
                FixedByFracOrig(speed, costable[angle]),
                FixedMul(speed,costable[angle]));
    ymove = DEMOCHOOSE_ORIG_SDL(
                -FixedByFracOrig(speed, sintable[angle]),
                -FixedMul(speed,sintable[angle]));

    ClipMove(player,xmove,ymove);

    player->tilex = (short)(player->x >> TILESHIFT);                // scale to tile values
    player->tiley = (short)(player->y >> TILESHIFT);

    offset = (player->tiley<<mapshift)+player->tilex;
    player->areanumber = *(mapsegs[0] + offset) -AREATILE;

    if (*(mapsegs[1] + offset) == EXITTILE)
        VictoryTile ();
   if (*(mapsegs[1] + offset) == EXITLEVEL)
   {
      playstate = ex_completed;
      gamestate.mapon = gamestate.mapon+1;
   }
   if (*(mapsegs[1] + offset) == BURN)
   {
   SD_PlaySound (TAKEDAMAGESND);
      TakeDamage (US_RndT()>>3,NULL);
   }

}


/*
=============================================================================

                                ACTIONS

=============================================================================
*/


/*
===============
=
= Cmd_Fire
=
===============
*/


void Cmd_Fire (void)
{
    if (gamestate.reloading) return;

 if (gamestate.mapon == 5)    //prevents player from shooting gun while using lamp must match other two
                     return;

    buttonheld[bt_attack] = true;

    gamestate.weaponframe = 0;

    player->state = &s_attack;

    gamestate.attackframe = 0;
    gamestate.attackcount =
        attackinfo[gamestate.weapon][gamestate.attackframe].tics;
    gamestate.weaponframe =
        attackinfo[gamestate.weapon][gamestate.attackframe].frame;
}

//===========================================================================

/*
===============
=
= Cmd_FirstAid //not used maybe redundant due to addition of Cmd_heal
=
===============
*/

void Cmd_FirstAid (void)
{
if (gamestate.medical > 0)
{
 SD_PlaySound (PERCENT100SND);
 gamestate.medical--;

if (gamestate.difficulty == gd_baby) HealSelf (40);
if (gamestate.difficulty == gd_easy) HealSelf (30);
if (gamestate.difficulty == gd_medium) HealSelf (25);
if (gamestate.difficulty == gd_hard) HealSelf (20);
}
}
/*
===============
=
= Cmd_Heal
=
===============
*/
void Cmd_Heal (void)
{
    if (gamestate.health == 100 ) return;    // return if full health of no medkits
    if (gamestate.medical == 0 ) return;

    buttonheld[bt_healthkit] = true;

 SD_PlaySound (PERCENT100SND);
 gamestate.medical--;

if (gamestate.difficulty == gd_baby) HealSelf (40);                          //leveldif
if (gamestate.difficulty == gd_easy) HealSelf (30);
if (gamestate.difficulty == gd_medium) HealSelf (25);
if (gamestate.difficulty == gd_hard) HealSelf (20);
}
/*
===============
=
= Cmd_Reload
=
===============
*/

void Cmd_Reload (void)
{
    if (gamestate.reloading) return;    // ignore command if already reloading

    if (!ClipSize(gamestate.weapon) || weaponinfo[gamestate.weapon].ammotype == ammo_none)
        return;    // weapon never reloads

    if (!CURRENTAMMO || CURRENTCLIP == ClipSize(gamestate.weapon))
        return;    // can't reload - either clip is full, or there's no ammo left in the reserve

    buttonheld[bt_reload] = true;

    gamestate.weaponframe = 0;

    player->state = &s_reload;

    gamestate.attackframe = 0;
    gamestate.attackcount =
        reloadinfo[gamestate.weapon][gamestate.attackframe].tics;
    gamestate.weaponframe =
        reloadinfo[gamestate.weapon][gamestate.attackframe].frame;
}


/*
===============
=
= Cmd_Use
=
===============
*/

void Cmd_Use (void)
{
    int     checkx,checky,doornum,dir;
    boolean elevatorok;

    //
    // find which cardinal direction the player is facing
    //
    if (player->angle < ANGLES/8 || player->angle > 7*ANGLES/8)
    {
        checkx = player->tilex + 1;
        checky = player->tiley;
        dir = di_east;
        elevatorok = true;
    }
    else if (player->angle < 3*ANGLES/8)
    {
        checkx = player->tilex;
        checky = player->tiley-1;
        dir = di_north;
        elevatorok = false;
    }
    else if (player->angle < 5*ANGLES/8)
    {
        checkx = player->tilex - 1;
        checky = player->tiley;
        dir = di_west;
        elevatorok = true;
    }
    else
    {
        checkx = player->tilex;
        checky = player->tiley + 1;
        dir = di_south;
        elevatorok = false;
    }

    doornum = tilemap[checkx][checky];
    if (*(mapsegs[1]+(checky<<mapshift)+checkx) == PUSHABLETILE)
    {
        //
        // pushable wall
        //

        PushWall (checkx,checky,dir);
        return;
    }
    if (!buttonheld[bt_use] && doornum == ELEVATORTILE && elevatorok)
    {
        //
        // use elevator
        //
        buttonheld[bt_use] = true;

        tilemap[checkx][checky]++;              // flip switch
        if (*(mapsegs[0]+(player->tiley<<mapshift)+player->tilex) == ALTELEVATORTILE)
            playstate = ex_secretlevel;
        else
            playstate = ex_completed;
        SD_PlaySound (LEVELDONESND);
        SD_WaitSoundDone();
    }
    else if (!buttonheld[bt_use] && doornum & 0x80)
    {
        buttonheld[bt_use] = true;
        OperateDoor (doornum & ~0x80);
    }
    //else
//        SD_PlaySound (DONOTHINGSND);
}

/*
=============================================================================

                                PLAYER CONTROL

=============================================================================
*/



/*
===============
=
= SpawnPlayer
=
===============
*/

void SpawnPlayer (int tilex, int tiley, int dir)
{
    player->obclass = playerobj;
    player->active = ac_yes;
    player->tilex = tilex;
    player->tiley = tiley;
    player->areanumber = (byte) *(mapsegs[0]+(player->tiley<<mapshift)+player->tilex);
    player->x = ((int32_t)tilex<<TILESHIFT)+TILEGLOBAL/2;
    player->y = ((int32_t)tiley<<TILESHIFT)+TILEGLOBAL/2;
    player->state = &s_player;
    player->angle = (1-dir)*90;
    if (player->angle<0)
        player->angle += ANGLES;
    player->flags = FL_NEVERMARK;
    Thrust (0,0);                           // set some variables

    InitAreas ();
}


//===========================================================================

/*
===============
=
= T_KnifeAttack
=
= Update player hands, and try to do damage when the proper frame is reached
=
===============
*/

void    KnifeAttack (objtype *ob)
{
    objtype *check,*closest;
    int32_t  dist;

    SD_PlaySound (ATKKNIFESND);
    // actually fire
    dist = 0x7fffffff;
    closest = NULL;
    for (check=ob->next; check; check=check->next)
    {
        if ( (check->flags & FL_SHOOTABLE) && (check->flags & FL_VISABLE)
            && abs(check->viewx-centerx) < shootdelta)
        {
            if (check->transx < dist)
            {
                dist = check->transx;
                closest = check;
            }
        }
    }

    if (!closest || dist > 0x18000l)
    {
        // missed
        return;
    }

    // hit something
    DamageActor (closest,US_RndT() >> 4);
}



#define COLSIZE 0x4000l
#define STEPDIST 0x800l

void    GunAttack (objtype *ob)
{
    switch (gamestate.weapon)
    {
        case wp_pistol:
            SD_PlaySound (ATKPISTOLSND);
            StartRecoilEffect(50);
            break;
        case wp_machinegun:
            SD_PlaySound (ATKMACHINEGUNSND);
            StartRecoilEffect(50);
            break;
        case wp_chaingun:
            SD_PlaySound (ATKGATLINGSND);
            StartRecoilEffect(50);
            break;
        case wp_emplacedgun:
            SD_PlaySound (ATKGATLINGSND);
            StartDamageShock (10);
            break;
    }

    madenoise = true;

    // BEGIN ZOOM BLOCK
   //StartRecoilEffect(50);
   // END ZOOM BLOCK


    int32_t         bulletx,bullety;
    short           bangle,damage,accuracy,numshots;
    boolean         hit;
    objtype         *check;

//
// AlumiuN's new weaponry code - trace bullets
//
    switch (gamestate.weapon)
    {
        case wp_pistol: accuracy = 3; numshots = 1; break;
        case wp_machinegun: accuracy = 5; numshots = 1; break;
        case wp_chaingun: accuracy = 9; numshots = 1; break;
        case wp_emplacedgun: accuracy = 14; numshots = 1; break;
    }

    for (int i = 0; i<numshots; i++)
    {
        bulletx = player->x;
        bullety = player->y;
        bangle = player->angle;

        if (accuracy)
        {
            if (US_RndT() > 127)
                bangle -= US_RndT()%(accuracy + 1);
            else
                bangle += US_RndT()%(accuracy + 1);
        }

        hit = false;    // Assume the worst! Oh, and reset the variable. :)

        while(1)
        {
            bulletx += FixedMul(STEPDIST,costable[bangle]);
            bullety -= FixedMul(STEPDIST,sintable[bangle]);

            //
            // check for solid walls
            //
            check = actorat[bulletx >> TILESHIFT][bullety >> TILESHIFT];
            if (check && !ISPOINTER(check) && (uintptr_t)check != 64)
            {

if (check && !ISPOINTER(check) && (uintptr_t)check != 64)
            {
            T_BulletPuff (bulletx,bullety,ob);
                hit = true;
            }
//                if ((uintptr_t)check < 128)      // Hit a wall
      //{
      // Add any effects that you want to happen when the bullet hits a wall
      //}
      //else if (doorposition[(uintptr_t)check - 128] < 0xdfff/2)
            //{
      // Add any effects that you want to happen when the bullet hits a door
      //}
              //  hit = true;
            }

            if (hit)
                break;

            check = objlist;
            while (check)
            {
                if (check->flags & FL_SHOOTABLE)
                {
                    if (LABS(bulletx - check->x) < COLSIZE && LABS(bullety - check->y) < COLSIZE)
                    {
                        hit = true;
                        switch (gamestate.weapon)
                        {
                            case wp_pistol: damage = 15 + US_RndT()%25; break;
                            case wp_machinegun: damage = 8 + US_RndT()%22; break;
                            case wp_chaingun: damage = 8 + US_RndT()%20; break;
                            case wp_emplacedgun: damage = 10 +US_RndT()%20; break;
                        }
                        DamageActor (check, damage);
                    }
                }
                check = check->next ;
            }

            if (hit)
                break;
        }
    }
}
//===========================================================================

/*
===============
=
= VictorySpin
=
===============
*/

void VictorySpin (void)
{
    int32_t    desty;

    if (player->angle > 270)
    {
        player->angle -= (short)(tics * 3);
        if (player->angle < 270)
            player->angle = 270;
    }
    else if (player->angle < 270)
    {
        player->angle += (short)(tics * 3);
        if (player->angle > 270)
            player->angle = 270;
    }

    desty = (((int32_t)player->tiley-5)<<TILESHIFT)-0x3000;

    if (player->y > desty)
    {
        player->y -= tics*4096;
        if (player->y < desty)
            player->y = desty;
    }
}


//===========================================================================

/*
===============
=
= T_Attack
=
===============
*/

void    T_Attack (objtype *ob)
{
    struct  atkinf  *cur;

    UpdateFace ();

    if (gamestate.victoryflag)              // watching the BJ actor
    {
        VictorySpin ();
        return;
    }

    if ( buttonstate[bt_use] && !buttonheld[bt_use] )
        buttonstate[bt_use] = false;

    if ( buttonstate[bt_attack] && !buttonheld[bt_attack])
        buttonstate[bt_attack] = false;

    ControlMovement (ob);
    if (gamestate.victoryflag)              // watching the BJ actor
        return;

    plux = (word) (player->x >> UNSIGNEDSHIFT);                     // scale to fit in unsigned
    pluy = (word) (player->y >> UNSIGNEDSHIFT);
    player->tilex = (short)(player->x >> TILESHIFT);                // scale to tile values
    player->tiley = (short)(player->y >> TILESHIFT);

    //
    // change frame and fire
    //
    gamestate.attackcount -= (short) tics;
    while (gamestate.attackcount <= 0)
    {
        cur = &attackinfo[gamestate.weapon][gamestate.attackframe];
        switch (cur->attack)
        {
            case -1:
                ob->state = &s_player;
                if (!WeaponAvailable(gamestate.weapon))
                {
                    gamestate.weapon = BestAvailableWeapon();
                    DrawWeapon ();
                    DrawAmmo ();
                }
                else
                {
                    if (gamestate.weapon != gamestate.chosenweapon && WeaponAvailable(gamestate.chosenweapon))
                    {
                        gamestate.weapon = gamestate.chosenweapon;
                        DrawWeapon ();
                        DrawAmmo ();
                    }
                }
                gamestate.attackframe = gamestate.weaponframe = 0;
                if (!CURRENTCLIP && ClipSize(gamestate.weapon) > 0 && CURRENTAMMO > 0)
                    Cmd_Reload ();
                return;

            case 4:
                if (!ReadyToFire(gamestate.weapon))
                    break;
                if (buttonstate[bt_attack])
                    gamestate.attackframe -= 2;
            case 1:
                if (!ReadyToFire(gamestate.weapon))
                {       // can only happen with chain gun
                    gamestate.attackframe++;
                    break;
                }
                GunAttack (ob);
                if (!ammocheat && weaponinfo[gamestate.weapon].ammotype != ammo_none)
                {
                   if(ClipSize(gamestate.weapon) != 9) //mounted start end
                   {
                       if (ClipSize(gamestate.weapon) > 0)
                          CURRENTCLIP--;
                       else
                          CURRENTAMMO--;
                   }
                }
                DrawWeapon ();
                DrawAmmo ();
                break;

            case 2:
                KnifeAttack (ob);
                break;

            case 3:
                if (ReadyToFire(gamestate.weapon) && buttonstate[bt_attack])
                    gamestate.attackframe -= 2;
                break;
        }

        gamestate.attackcount += cur->tics;
        gamestate.attackframe++;
        gamestate.weaponframe =
            attackinfo[gamestate.weapon][gamestate.attackframe].frame;
    }
}

//===========================================================================

/*
===============
=
= T_Reload
=
===============
*/

void    T_Reload (objtype *ob)
{
    struct  rloadinf  *cur;

    UpdateFace ();

    if (gamestate.victoryflag)              // watching the BJ actor
    {
        VictorySpin ();
        return;
    }

    ControlMovement (ob);
    if (gamestate.victoryflag)              // watching the BJ actor
        return;

    plux = (word) (player->x >> UNSIGNEDSHIFT);                     // scale to fit in unsigned
    pluy = (word) (player->y >> UNSIGNEDSHIFT);
    player->tilex = (short)(player->x >> TILESHIFT);                // scale to tile values
    player->tiley = (short)(player->y >> TILESHIFT);

    buttonheld[bt_attack] = false;
    buttonstate[bt_attack] = false;

    gamestate.reloading = true;

    //
    // change frame and reload
    //
    gamestate.attackcount -= (short) tics;
    while (gamestate.attackcount <= 0)
    {
        cur = &reloadinfo[gamestate.weapon][gamestate.attackframe];
        switch (cur->attack)
        {
            case -1:    // go back to first frame
                ob->state = &s_player;
                gamestate.reloading = false;
                gamestate.attackframe = gamestate.weaponframe = 0;
                return;

            case 1:     // play a "lock and load" sound
                switch (gamestate.weapon)
                {
                 case wp_pistol:
                    SD_PlaySound (MEINSND);       //reload sound
                    break;
                 case wp_machinegun:
                    SD_PlaySound (ROSESND);
                    break;
                 case wp_chaingun:
                    SD_PlaySound (ROSESND);
                    break;
                 case wp_emplacedgun:
                    SD_PlaySound (ROSESND);
                    break;
                }
                break;

            case 2:    // reload a full clip
                ReloadWeapon (ClipSize(gamestate.weapon)-CURRENTCLIP);
                break;

            case 3:    // reload 1 round
                ReloadWeapon (1);
                break;

            case 4:    // go back 2 frames if clip isn't full yet
                if (CURRENTCLIP < ClipSize(gamestate.weapon) && CURRENTAMMO > 0)
                    gamestate.attackframe -= 2;
                break;
        }

        gamestate.attackcount += cur->tics;
        gamestate.attackframe++;
        gamestate.weaponframe =
            reloadinfo[gamestate.weapon][gamestate.attackframe].frame;
    }
}


/*
===============
=
= ReloadWeapon
=
===============
*/

void    ReloadWeapon (int ammo)
{
    if (!CURRENTAMMO)
        return;

    if (ammo > CURRENTAMMO)
        ammo = CURRENTAMMO;

    CURRENTCLIP += ammo;
    CURRENTAMMO -= ammo;

    DrawWeapon ();
    DrawAmmo ();
}



/*
===============
=
= T_Player
=
===============
*/

void    T_Player (objtype *ob)
{
    if (gamestate.victoryflag)              // watching the BJ actor
    {
        VictorySpin ();
        return;
    }

    UpdateFace ();
    CheckWeaponChange ();

    if ( buttonstate[bt_use] )
        Cmd_Use ();

    if ( buttonstate[bt_attack] && !buttonheld[bt_attack])
        Cmd_Fire ();

    if ( buttonstate[bt_reload] && !buttonheld[bt_reload])
        Cmd_Reload ();

    if ( buttonstate[bt_healthkit] && !buttonheld[bt_healthkit])
        Cmd_Heal ();

        if (Keyboard[sc_I])
        {
            if (gamestate.weapon == wp_emplacedgun) return;
            if (gamestate.weapon == wp_knife) return;
            gamestate.ironsights = gamestate.ironsights == true ? false : true;
            Keyboard[sc_I] = false;
        }

/*        if (gamestate.weapon == wp_emplacedgun) //mounted start ?? is it needed ??
        {
        if (buttonstate[bt_use])
        {
        gamestate.clip[4] = 0;
        gamestate.ammo[ammo_emplaced] = 0;
        }
        buttonstate[bt_use] = false; //mounted end
        } */


/*    if ( Keyboard[sc_H] )  //|| ci.button3) // ci not declared in this scope
    {
         if (gamestate.health > 99)
         return;
         else
         {
         Cmd_FirstAid ();
         Keyboard[sc_H] = false;
     }
}*/

    ControlMovement (ob);
    if (gamestate.victoryflag)              // watching the BJ actor
        return;

    plux = (word) (player->x >> UNSIGNEDSHIFT);                     // scale to fit in unsigned
    pluy = (word) (player->y >> UNSIGNEDSHIFT);
    player->tilex = (short)(player->x >> TILESHIFT);                // scale to tile values
    player->tiley = (short)(player->y >> TILESHIFT);
}
