// WL_DRAW.C

#include "wl_def.h"
#pragma hdrstop

#include "wl_cloudsky.h"
#include "wl_atmos.h"
#include "wl_shade.h"

/*
=============================================================================

                               LOCAL CONSTANTS

=============================================================================
*/

// the door is the last picture before the sprites
#define DOORWALL        (PMSpriteStart-8)

#define ACTORSIZE       0x4000
#define TRUCKTILE1      53
#define TRUCKTILE2      55
#define TRUCKTILE3      1 

/*
=============================================================================

                              GLOBAL VARIABLES

=============================================================================
*/

// BEGIN ZOOM BLOCK
// Global definitions
#ifndef   FOCALLENGTH
#define   FOCALLENGTH            0x5700L               // MUST match the FOCALLENGTH definition in WL_MAIN.C
#endif
#define   FOCALTHRESHOLD         FOCALLENGTH            // Greater lengths won't change the view position
// END ZOOM BLOCK

static byte *vbuf = NULL;
unsigned vbufPitch = 0;

int32_t    lasttimecount;
int32_t    frameon;
//  Added line below for FPS Counter
int        fps_frames=0, fps_time=0, fps=0; 
int        messagetime = 0;
int        messagetime1 = 0;
int        messagetime2 = 0;
boolean fpscounter;


int *wallheight;
int min_wallheight;

//
// math tables
//
short *pixelangle;
int32_t finetangent[FINEANGLES/4];
fixed sintable[ANGLES+ANGLES/4];
fixed *costable = sintable+(ANGLES/4);

//
// refresh variables
//
fixed   viewx,viewy;                    // the focal point
short   viewangle;
fixed   viewsin,viewcos;

void    TransformActor (objtype *ob);
void    BuildTables (void);
void    ClearScreen (void);
int     CalcRotate (objtype *ob);
void    DrawScaleds (void);
void    CalcTics (void);
void    ThreeDRefresh (void);



//
// wall optimization variables
//
int     lastside;               // true for vertical
int32_t    lastintercept;
int     lasttilehit;
int     lasttexture;

//
// ray tracing variables
//
short    focaltx,focalty,viewtx,viewty;
longword xpartialup,xpartialdown,ypartialup,ypartialdown;

short   midangle,angle;

word    tilehit;
int     pixx;

short   xtile,ytile;
short   xtilestep,ytilestep;
int32_t    xintercept,yintercept;
word    xstep,ystep;
word    xspot,yspot;
int     texdelta;

word horizwall[MAXWALLTILES],vertwall[MAXWALLTILES];


/*
============================================================================

                           3 - D  DEFINITIONS

============================================================================
*/

/*
========================
=
= TransformActor
=
= Takes paramaters:
=   gx,gy               : globalx/globaly of point
=
= globals:
=   viewx,viewy         : point of view
=   viewcos,viewsin     : sin/cos of viewangle
=   scale               : conversion from global value to screen value
=
= sets:
=   screenx,transx,transy,screenheight: projected edge location and size
=
========================
*/


//
// transform actor
//
void TransformActor (objtype *ob)
{
    fixed gx,gy,gxt,gyt,nx,ny;

//
// translate point to view centered coordinates
//
    gx = ob->x-viewx;
    gy = ob->y-viewy;

//
// calculate newx
//
    gxt = FixedMul(gx,viewcos);
    gyt = FixedMul(gy,viewsin);
    nx = gxt-gyt-ACTORSIZE;         // fudge the shape forward a bit, because
                                    // the midpoint could put parts of the shape
                                    // into an adjacent wall

//
// calculate newy
//
    gxt = FixedMul(gx,viewsin);
    gyt = FixedMul(gy,viewcos);
    ny = gyt+gxt;

//
// calculate perspective ratio
//
    ob->transx = nx;
    ob->transy = ny;

    if (nx<MINDIST)                 // too close, don't overflow the divide
    {
        ob->viewheight = 0;
        return;
    }

    ob->viewx = (word)(centerx + ny*scale/nx);

//
// calculate height (heightnumerator/(nx>>8))
//
    ob->viewheight = (word)(heightnumerator/(nx>>8));
}

//==========================================================================

/*
========================
=
= TransformTile
=
= Takes paramaters:
=   tx,ty               : tile the object is centered in
=
= globals:
=   viewx,viewy         : point of view
=   viewcos,viewsin     : sin/cos of viewangle
=   scale               : conversion from global value to screen value
=
= sets:
=   screenx,transx,transy,screenheight: projected edge location and size
=
= Returns true if the tile is withing getting distance
=
========================
*/

boolean TransformTile (int tx, int ty, short *dispx, short *dispheight)
{
    fixed gx,gy,gxt,gyt,nx,ny;

//
// translate point to view centered coordinates
//
    gx = ((int32_t)tx<<TILESHIFT)+0x8000-viewx;
    gy = ((int32_t)ty<<TILESHIFT)+0x8000-viewy;

//
// calculate newx
//
    gxt = FixedMul(gx,viewcos);
    gyt = FixedMul(gy,viewsin);
    nx = gxt-gyt-0x2000;            // 0x2000 is size of object

//
// calculate newy
//
    gxt = FixedMul(gx,viewsin);
    gyt = FixedMul(gy,viewcos);
    ny = gyt+gxt;


//
// calculate height / perspective ratio
//
    if (nx<MINDIST)                 // too close, don't overflow the divide
        *dispheight = 0;
    else
    {
        *dispx = (short)(centerx + ny*scale/nx);
        *dispheight = (short)(heightnumerator/(nx>>8));
    }

//
// see if it should be grabbed
//
    if (nx<TILEGLOBAL && ny>-TILEGLOBAL/2 && ny<TILEGLOBAL/2)
        return true;
    else
        return false;
}

//==========================================================================

/*
====================
=
= CalcHeight
=
= Calculates the height of xintercept,yintercept from viewx,viewy
=
====================
*/

int CalcHeight()
{
    fixed z = FixedMul(xintercept - viewx, viewcos)
        - FixedMul(yintercept - viewy, viewsin);
    if(z < MINDIST) z = MINDIST;
    int height = heightnumerator / (z >> 8);
    if(height < min_wallheight) min_wallheight = height;
    return height;
}

//==========================================================================

/*
===================
=
= ScalePost
=
===================
*/

byte *postsource;
int postx;
int postwidth;

void ScalePost()
{
    int ywcount, yoffs, yw, yd, yendoffs;
    byte col;

#ifdef USE_SHADING
    byte *curshades = shadetable[GetShade(wallheight[postx])];
#endif

    ywcount = yd = wallheight[postx] >> 3;
    if(yd <= 0) yd = 100;

    yoffs = (viewheight / 2 - ywcount) * vbufPitch;
    if(yoffs < 0) yoffs = 0;
    yoffs += postx;

    yendoffs = viewheight / 2 + ywcount - 1;
    yw=TEXTURESIZE-1;

    while(yendoffs >= viewheight)
    {
        ywcount -= TEXTURESIZE/2;
        while(ywcount <= 0)
        {
            ywcount += yd;
            yw--;
        }
        yendoffs--;
    }
    if(yw < 0) return;

#ifdef USE_SHADING
    col = curshades[postsource[yw]];
#else
    col = postsource[yw];
#endif
    yendoffs = yendoffs * vbufPitch + postx;
    while(yoffs <= yendoffs)
    {
        vbuf[yendoffs] = col;
        ywcount -= TEXTURESIZE/2;
        if(ywcount <= 0)
        {
            do
            {
                ywcount += yd;
                yw--;
            }
            while(ywcount <= 0);
            if(yw < 0) break;
#ifdef USE_SHADING
            col = curshades[postsource[yw]];
#else
            col = postsource[yw];
#endif
        }
        yendoffs -= vbufPitch;
    }
}

void GlobalScalePost(byte *vidbuf, unsigned pitch)
{
    vbuf = vidbuf;
    vbufPitch = pitch;
    ScalePost();
}

#ifdef WALLPATCH

/*******************
**
** ApplyPatch
** Thanks to Adam Biser(big thanks for helping with bugs) and Hassboy
********************/
byte   patchedwall[PMPageSize];
int      lastpatchnum;
#define FIRSTPATCHOBJ    500
#define NUMPATCHES       81//89   // This is the number of patch pairs (light/dark) in use.
#define PATCHSTART       DOORWALL-NUMPATCHES*2

unsigned GetPatchNum(boolean useXspot)
{
    // work off xspot/yspot, instead of xtile/ytile
    unsigned spot, x, y;
    //(useXspot? spot=xspot : spot=yspot);
    spot = useXspot ? xspot : yspot; 

    x = (unsigned)((spot+1)/64);
    y = (unsigned)((spot+1)%64) -1;
   
    spot = MAPSPOT(x,y,1);
// Restrict patch objects to the given range.
   if (spot < FIRSTPATCHOBJ || spot >= FIRSTPATCHOBJ+NUMPATCHES )
      return 0;
   else
      return spot;

}

boolean ApplyPatch(int wallpic, boolean useXspot)
{
      byte   *scan;
      int    pixel;
      unsigned spot;

   spot = GetPatchNum(useXspot);
   if (spot > 0 && wallpic < DOORWALL) // Don't do door-related textures.
   {
      // Keep track of the patch in memory.
      //   The wall is already stored elsewhere.
      lastpatchnum = spot;
      spot -= FIRSTPATCHOBJ;
      // Adjust for light/dark patch sides
      spot = spot * 2 + (lastside?1:0);
      // Load the wall into the byte array first.
      scan = PM_GetTexture(wallpic);
      for (pixel=0; pixel<PMPageSize; pixel++,scan++) {
       patchedwall[pixel] = *scan;
       }
       // Now add the patch. This MUST be done separate from wallpic's page.
       scan = PM_GetTexture(PATCHSTART+spot);
       for (pixel=0; pixel<PMPageSize; pixel++,scan++) {
       if (*scan < 255) {
        patchedwall[pixel] = *scan;
    }
}       return true;
   }
   lastpatchnum = 0;
   return false;
}

#endif

/*
====================
=
= HitVertWall
=
= tilehit bit 7 is 0, because it's not a door tile
= if bit 6 is 1 and the adjacent tile is a door tile, use door side pic
=
====================
*/

void HitVertWall (void)
{
    int wallpic;
    int texture;

    texture = ((yintercept+texdelta)>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    if (xtilestep == -1)
    {
//        texture = TEXTUREMASK-texture;
//        xintercept += TILEGLOBAL;
     if(tilehit!=TRUCKTILE1 && tilehit!=TRUCKTILE2 && tilehit!=TRUCKTILE3) texture = 0xfc0-texture;
      xintercept += TILEGLOBAL;

    }
#ifdef WALLPATCH
    if(lastside==1 && lastintercept==xtile && lasttilehit==tilehit && !(lasttilehit & 0x40) && GetPatchNum(true)==lastpatchnum)
//    if(lastside==1 && lastintercept==ytile && lasttilehit==tilehit && !(lasttilehit & 0x40) && lastpatchnum==GetPatchNum(false)) 
#else
    if(lastside==1 && lastintercept==xtile && lasttilehit==tilehit && !(lasttilehit & 0x40))
#endif
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx = pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=1;
    lastintercept=xtile;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();

//switch (lasttilehit)
//   {
//   case 14: // on map wall ID is 2
//   case 12: // on map wall ID is 42
//      tilehit+=(frameon>>3)%4; // the 3 is speed, the 4 is frames
//      break;
//   } 




    postx = pixx;
    postwidth = 1;

    if (tilehit & 0x40)
    {                                                               // check for adjacent doors
        ytile = (short)(yintercept>>TILESHIFT);
        if ( tilemap[xtile-xtilestep][ytile]&0x80 )
            wallpic = DOORWALL+3;
        else
            wallpic = vertwall[tilehit & ~0x40];
    }
    else
        wallpic = vertwall[tilehit];

#ifdef WALLPATCH
        //AssignWall (wallpic);
        if (ApplyPatch (wallpic, true))
                postsource = patchedwall+texture;
        else
                postsource = PM_GetTexture(wallpic)+texture;
#else
    postsource = PM_GetTexture(wallpic) + texture;
#endif 
}
 

/*
====================
=
= HitHorizWall
=
= tilehit bit 7 is 0, because it's not a door tile
= if bit 6 is 1 and the adjacent tile is a door tile, use door side pic
=
====================
*/

void HitHorizWall (void)
{
    int wallpic;
    int texture;
   
    texture = ((xintercept+texdelta)>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
   if (ytilestep == -1)
      yintercept += TILEGLOBAL;
   else if(tilehit!=TRUCKTILE1 && tilehit!=TRUCKTILE2 && tilehit!=TRUCKTILE3)
      texture = 0xfc0-texture; 
//    if (ytilestep == -1)
//        yintercept += TILEGLOBAL;
//    else
//        texture = TEXTUREMASK-texture;

#ifdef WALLPATCH
    if(lastside==1 && lastintercept==ytile && lasttilehit==tilehit && !(lasttilehit & 0x40) && lastpatchnum==GetPatchNum(false))
#else
    if(lastside==1 && lastintercept==ytile && lasttilehit==tilehit && !(lasttilehit & 0x40))
#endif
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=0;
    lastintercept=ytile;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;

    if (tilehit & 0x40)
    {                                                               // check for adjacent doors
        xtile = (short)(xintercept>>TILESHIFT);
        if ( tilemap[xtile][ytile-ytilestep]&0x80)
            wallpic = DOORWALL+2;
        else
            wallpic = horizwall[tilehit & ~0x40];
    }
    else
        wallpic = horizwall[tilehit];

#ifdef WALLPATCH
        //AssignWall (wallpic);
        if (ApplyPatch (wallpic, false))
                postsource = patchedwall+texture;
        else
                postsource = PM_GetTexture(wallpic)+texture;
#else
    postsource = PM_GetTexture(wallpic) + texture;
#endif
}//==========================================================================

/*
====================
=
= HitHorizDoor
=
====================
*/

void HitHorizDoor (void)
{
    int doorpage;
    int doornum;
    int texture;

    doornum = tilehit&0x7f;
    texture = ((xintercept-doorposition[doornum])>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;

    if(lasttilehit==tilehit)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=2;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;

    switch(doorobjlist[doornum].lock)
    {
        case dr_normal:
            doorpage = DOORWALL;
            break;
        case dr_lock1:
        case dr_lock2:
        case dr_lock3:
        case dr_lock4:
            doorpage = DOORWALL+6;
            break;
        case dr_elevator:
            doorpage = DOORWALL+4;
            break;
    }

    postsource = PM_GetTexture(doorpage) + texture;
}

//==========================================================================

/*
====================
=
= HitVertDoor
=
====================
*/

void HitVertDoor (void)
{
    int doorpage;
    int doornum;
    int texture;

    doornum = tilehit&0x7f;
    texture = ((yintercept-doorposition[doornum])>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;

    if(lasttilehit==tilehit)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=2;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;

    switch(doorobjlist[doornum].lock)
    {
        case dr_normal:
            doorpage = DOORWALL+1;
            break;
        case dr_lock1:
        case dr_lock2:
        case dr_lock3:
        case dr_lock4:
            doorpage = DOORWALL+7;
            break;
        case dr_elevator:
            doorpage = DOORWALL+5;
            break;
    }

    postsource = PM_GetTexture(doorpage) + texture;
}

//==========================================================================


/*
====================
=
= HitHorizPWall
=
= A pushable wall in action has been hit
=
====================
*/

void HitHorizPWall (void)
{
    int wallpic;
    int texture,offset;

    texture = (xintercept>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    offset = pwallpos<<10;
    if (ytilestep == -1)
        yintercept += TILEGLOBAL-offset;
    else
    {
        texture = TEXTUREMASK-texture;
        yintercept += offset;
    }

    if(lasttilehit==tilehit && lastside==0)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=0;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;

    wallpic = horizwall[pwalltile&63];

    postsource = PM_GetTexture(wallpic) + texture;
}

/*
====================
=
= HitVertPWall
=
= A pushable wall in action has been hit
=
====================
*/

void HitVertPWall (void)
{
    int wallpic;
    int texture,offset;

    texture = (yintercept>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    offset = pwallpos<<10;
    if (xtilestep == -1)
    {
        xintercept += TILEGLOBAL-offset;
        texture = TEXTUREMASK-texture;
    }
    else
        xintercept += offset;

    if(lasttilehit==tilehit && lastside==1)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=1;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;

    wallpic = vertwall[pwalltile&63];

    postsource = PM_GetTexture(wallpic) + texture;
}

#define HitHorizBorder HitHorizWall
#define HitVertBorder HitVertWall

//==========================================================================

byte vgaCeiling[]=
{
#ifndef SPEAR
 0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0xbf,
 0x4e,0x4e,0x4e,0x1d,0x8d,0x4e,0x1d,0x2d,0x1d,0x8d,
 0x1d,0x1d,0x1d,0x1d,0x1d,0x2d,0xdd,0x1d,0x1d,0x98,

 0x1d,0x9d,0x2d,0xdd,0xdd,0x9d,0x2d,0x4d,0x1d,0xdd,
 0x7d,0x1d,0x2d,0x2d,0xdd,0xd7,0x1d,0x1d,0x1d,0x2d,
 0x1d,0x1d,0x1d,0x1d,0xdd,0xdd,0x7d,0xdd,0xdd,0xdd
#else
 0x6f,0x4f,0x1d,0xde,0xdf,0x2e,0x7f,0x9e,0xae,0x7f,
 0x1d,0xde,0xdf,0xde,0xdf,0xde,0xe1,0xdc,0x2e,0x1d,0xdc
#endif
};

/*
=====================
=
= VGAClearScreen
=
=====================
*/

void VGAClearScreen (void)
{
    byte ceiling=vgaCeiling[gamestate.episode*10+mapon];

    int y;
    byte *ptr = vbuf;
#ifdef USE_SHADING
    for(y = 0; y < viewheight / 2; y++, ptr += vbufPitch)
        memset(ptr, shadetable[GetShade((viewheight / 2 - y) << 3)][ceiling], viewwidth);
    for(; y < viewheight; y++, ptr += vbufPitch)
        memset(ptr, shadetable[GetShade((y - viewheight / 2) << 3)][0x19], viewwidth);
#else
    for(y = 0; y < viewheight / 2; y++, ptr += vbufPitch)
        memset(ptr, ceiling, viewwidth);
    for(; y < viewheight; y++, ptr += vbufPitch)
        memset(ptr, 0x19, viewwidth);
#endif
}

//==========================================================================

/*
=====================
=
= CalcRotate
=
=====================
*/

int CalcRotate (objtype *ob)
{
    int angle, viewangle;

    // this isn't exactly correct, as it should vary by a trig value,
    // but it is close enough with only eight rotations

    viewangle = player->angle + (centerx - ob->viewx)/8;

    if (ob->obclass == rocketobj || ob->obclass == hrocketobj)
        angle = (viewangle-180) - ob->angle;
    else
        angle = (viewangle-180) - dirangle[ob->dir];

    angle+=ANGLES/16;
    while (angle>=ANGLES)
        angle-=ANGLES;
    while (angle<0)
        angle+=ANGLES;

    if (ob->state->rotate == 2)             // 2 rotation pain frame
        return 0;               // pain with shooting frame bugfix

    return angle/(ANGLES/8);
}

void ScaleShape (int xcenter, int shapenum, unsigned height, uint32_t flags)
{
    t_compshape *shape;
    unsigned scale,pixheight;
    unsigned starty,endy;
    word *cmdptr;
    byte *cline;
    byte *line;
    byte *vmem;
    int actx,i,upperedge;
    short newstart;
    int scrstarty,screndy,lpix,rpix,pixcnt,ycnt;
    unsigned j;
    byte col;

#ifdef USE_SHADING
    byte *curshades;
    if(flags & FL_FULLBRIGHT)
        curshades = shadetable[0];
    else
        curshades = shadetable[GetShade(height)];
#endif

    shape = (t_compshape *) PM_GetSprite(shapenum);

    scale=height>>3;                 // low three bits are fractional
    if(!scale) return;   // too close or far away

    pixheight=scale*SPRITESCALEFACTOR;
    actx=xcenter-scale;
    upperedge=viewheight/2-scale;

    cmdptr=(word *) shape->dataofs;

    for(i=shape->leftpix,pixcnt=i*pixheight,rpix=(pixcnt>>6)+actx;i<=shape->rightpix;i++,cmdptr++)
    {
        lpix=rpix;
        if(lpix>=viewwidth) break;
        pixcnt+=pixheight;
        rpix=(pixcnt>>6)+actx;
        if(lpix!=rpix && rpix>0)
        {
            if(lpix<0) lpix=0;
            if(rpix>viewwidth) rpix=viewwidth,i=shape->rightpix+1;
            cline=(byte *)shape + *cmdptr;
            while(lpix<rpix)
            {
                if(wallheight[lpix]<=(int)height)
                {
                    line=cline;
                    while((endy = READWORD(line)) != 0)
                    {
                        endy >>= 1;
                        newstart = READWORD(line);
                        starty = READWORD(line) >> 1;
                        j=starty;
                        ycnt=j*pixheight;
                        screndy=(ycnt>>6)+upperedge;
                        if(screndy<0) vmem=vbuf+lpix;
                        else vmem=vbuf+screndy*vbufPitch+lpix;
                        for(;j<endy;j++)
                        {
                            scrstarty=screndy;
                            ycnt+=pixheight;
                            screndy=(ycnt>>6)+upperedge;
                            if(scrstarty!=screndy && screndy>0)
                            {
#ifdef USE_SHADING
                                col=curshades[((byte *)shape)[newstart+j]];
#else
                                col=((byte *)shape)[newstart+j];
#endif
                                if(scrstarty<0) scrstarty=0;
                                if(screndy>viewheight) screndy=viewheight,j=endy;

                                while(scrstarty<screndy)
                                {
                                    *vmem=col;
                                    vmem+=vbufPitch;
                                    scrstarty++;
                                }
                            }
                        }
                    }
                }
                lpix++;
            }
        }
    }
}

void SimpleScaleShape (int xcenter, int shapenum, unsigned height)
{
    t_compshape   *shape;
    unsigned scale,pixheight;
    unsigned starty,endy;
    word *cmdptr;
    byte *cline;
    byte *line;
    int actx,i,upperedge;
    short newstart;
    int scrstarty,screndy,lpix,rpix,pixcnt,ycnt;
    unsigned j;
    byte col;
    byte *vmem;

    shape = (t_compshape *) PM_GetSprite(shapenum);

    scale=height>>1;
    pixheight=scale*SPRITESCALEFACTOR;
    actx=xcenter-scale;
    upperedge=viewheight/2-scale;

    cmdptr=shape->dataofs;

    for(i=shape->leftpix,pixcnt=i*pixheight,rpix=(pixcnt>>6)+actx;i<=shape->rightpix;i++,cmdptr++)
    {
        lpix=rpix;
        if(lpix>=viewwidth) break;
        pixcnt+=pixheight;
        rpix=(pixcnt>>6)+actx;
        if(lpix!=rpix && rpix>0)
        {
            if(lpix<0) lpix=0;
            if(rpix>viewwidth) rpix=viewwidth,i=shape->rightpix+1;
            cline = (byte *)shape + *cmdptr;
            while(lpix<rpix)
            {
                line=cline;
                while((endy = READWORD(line)) != 0)
                {
                    endy >>= 1;
                    newstart = READWORD(line);
                    starty = READWORD(line) >> 1;
                    j=starty;
                    ycnt=j*pixheight;
                    screndy=(ycnt>>6)+upperedge;
                    if(screndy<0) vmem=vbuf+lpix;
                    else vmem=vbuf+screndy*vbufPitch+lpix;
                    for(;j<endy;j++)
                    {
                        scrstarty=screndy;
                        ycnt+=pixheight;
                        screndy=(ycnt>>6)+upperedge;
                        if(scrstarty!=screndy && screndy>0)
                        {
                            col=((byte *)shape)[newstart+j];
                            if(scrstarty<0) scrstarty=0;
                            if(screndy>viewheight) screndy=viewheight,j=endy;

                            while(scrstarty<screndy)
                            {
                                *vmem=col;
                                vmem+=vbufPitch;
                                scrstarty++;
                            }
                        }
                    }
                }
                lpix++;
            }
        }
    }
}

/*
=====================
=
= DrawScaleds
=
= Draws all objects that are visable
=
=====================
*/

#define MAXVISABLE 250

typedef struct
{
    short      viewx,
               viewheight,
               shapenum;
    short      flags;          // this must be changed to uint32_t, when you
                               // you need more than 16-flags for drawing
#ifdef USE_DIR3DSPR
    statobj_t *transsprite;
#endif
} visobj_t;

visobj_t vislist[MAXVISABLE];
visobj_t *visptr,*visstep,*farthest;

void DrawScaleds (void)
{
    int      i,least,numvisable,height;
    byte     *tilespot,*visspot;
    unsigned spotloc;

    statobj_t *statptr;
    objtype   *obj;

    visptr = &vislist[0];

// Animated Objects code
    frameon+=tics;
    frameon%=2520;  // remember 70 tics/second
// End Animated Objects code



//
// place static objects
//
    for (statptr = &statobjlist[0] ; statptr !=laststatobj ; statptr++)
    {
        if ((visptr->shapenum = statptr->shapenum) == -1)
            continue;                                               // object has been deleted

        if (!*statptr->visspot)
            continue;                                               // not visable

        if (TransformTile (statptr->tilex,statptr->tiley,
            &visptr->viewx,&visptr->viewheight) && statptr->flags & FL_BONUS)
        {
            GetBonus (statptr);
            if(statptr->shapenum == -1)
                continue;                                           // object has been taken
        }

        if (!visptr->viewheight)
            continue;                                               // to close to the object



      switch (statptr->shapenum)
      {
      case SPR_STAT_29://fire
         visptr->shapenum+=(frameon>>4)%3; // slow animation with 3 frames - animated fire
         break;
      case SPR_STAT_45://heater
         visptr->shapenum+=(frameon>>5)%2; // slow animation with 2 frames
         break;
//      case SPR_STAT_29:
//         visptr->shapenum+=(frameon>>2)%4; // fast animation with 4 frames
//         break;
      } 

#ifdef USE_DIR3DSPR
        if(statptr->flags & FL_DIR_MASK)
            visptr->transsprite=statptr;
        else
            visptr->transsprite=NULL;
#endif

        if (visptr < &vislist[MAXVISABLE-1])    // don't let it overflow
        {
            visptr->flags = (short) statptr->flags;
            visptr++;
        }
    }

//
// place active objects
//
    for (obj = player->next;obj;obj=obj->next)
    {
        if ((visptr->shapenum = obj->state->shapenum)==0)
            continue;                                               // no shape

        spotloc = (obj->tilex<<mapshift)+obj->tiley;   // optimize: keep in struct?
        visspot = &spotvis[0][0]+spotloc;
        tilespot = &tilemap[0][0]+spotloc;

        //
        // could be in any of the nine surrounding tiles
        //
        if (*visspot
            || ( *(visspot-1) && !*(tilespot-1) )
            || ( *(visspot+1) && !*(tilespot+1) )
            || ( *(visspot-65) && !*(tilespot-65) )
            || ( *(visspot-64) && !*(tilespot-64) )
            || ( *(visspot-63) && !*(tilespot-63) )
            || ( *(visspot+65) && !*(tilespot+65) )
            || ( *(visspot+64) && !*(tilespot+64) )
            || ( *(visspot+63) && !*(tilespot+63) ) )
        {
            obj->active = ac_yes;
            TransformActor (obj);
            if (!obj->viewheight)
                continue;                                               // too close or far away

            visptr->viewx = obj->viewx;
            visptr->viewheight = obj->viewheight;
            if (visptr->shapenum == -1)
                visptr->shapenum = obj->temp1;  // special shape

            if (obj->state->rotate)
                visptr->shapenum += CalcRotate (obj);

            if (visptr < &vislist[MAXVISABLE-1])    // don't let it overflow
            {
                visptr->flags = (short) obj->flags;
#ifdef USE_DIR3DSPR
                visptr->transsprite = NULL;
#endif
                visptr++;
            }
            obj->flags |= FL_VISABLE;
        }
        else
            obj->flags &= ~FL_VISABLE;
    }

//
// draw from back to front
//
    numvisable = (int) (visptr-&vislist[0]);

    if (!numvisable)
        return;                                                                 // no visable objects

    for (i = 0; i<numvisable; i++)
    {
        least = 32000;
        for (visstep=&vislist[0] ; visstep<visptr ; visstep++)
        {
            height = visstep->viewheight;
            if (height < least)
            {
                least = height;
                farthest = visstep;
            }
        }
        //
        // draw farthest
        //
#ifdef USE_DIR3DSPR
        if(farthest->transsprite)
            Scale3DShape(vbuf, vbufPitch, farthest->transsprite);
        else
#endif
            ScaleShape(farthest->viewx, farthest->shapenum, farthest->viewheight, farthest->flags);

        farthest->viewheight = 32000;
    }
}

//==========================================================================

/*
==============
=
= DrawPlayerWeapon
=
= Draw the player's hands
=
==============
*/
int weaponscale[NUMWEAPONS] = {SPR_KNIFEREADY, SPR_PISTOLREADY,
    SPR_MACHINEGUNREADY, SPR_CHAINREADY, SPR_EMPLACEDREADY};

int reloadscale[NUMWEAPONS] = {SPR_KNIFEREADY, SPR_PISTOL_RLOAD1,
    SPR_MACHINEGUN_RLOAD1, SPR_CHAINGUN_RLOAD1, SPR_PISTOL_RLOAD1}; 

int ironsightscale[NUMWEAPONS] = {SPR_KNIFEREADY, SPR_IRONSIGHT1,
    SPR_IRONSIGHT2, SPR_IRONSIGHT3, SPR_EMPLACEDREADY}; 


void DrawPlayerWeapon (void)
{
    int shapenum;

#ifndef SPEAR
    if (gamestate.victoryflag)
    {
#ifndef APOGEE_1_0
        if (player->state == &s_deathcam && (GetTimeCount()&32) )
            SimpleScaleShape(viewwidth - (viewwidth/3),SPR_DEATHCAM,viewheight+1);
#endif
        return;
    }
#endif

if (gamestate.weapon == wp_knife) {
ResetZoomEffects();
gamestate.ironsights = false; }

      //remember there are four?? that must be changed for the lamp
 if (gamestate.mapon == 5)// draw lamp on level five, the cave of wonders! Oh'ahh!!
 {
    if (gamestate.TimeCount%4)
        SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,SPR_LAMP1,viewheight+1+gamestate.bobber);
    if (gamestate.TimeCount%5)
        SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,SPR_LAMP2,viewheight+1+gamestate.bobber);
    if (gamestate.TimeCount%3)
        SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,SPR_LAMP3,viewheight+1+gamestate.bobber);
    if (gamestate.TimeCount%7)
        SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,SPR_LAMP4,viewheight+1+gamestate.bobber);
    else
        SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,SPR_LAMP5,viewheight+1+gamestate.bobber);
 }
else
{
      if (gamestate.weapon != -1)
      {
        if (gamestate.reloading) 
        { 
           shapenum = reloadscale[gamestate.weapon]+gamestate.weaponframe;
           SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,shapenum,viewheight+1+gamestate.bobber);
        }
        else 
        {
             if (gamestate.weapon == wp_emplacedgun)
                 SimpleScaleShape((viewwidth/2)+(gamestate.bobber2/10),(weaponscale[gamestate.weapon]+gamestate.weaponframe),viewheight+1+(gamestate.bobber/10));
             else
             {
                 //shapenum = weaponscale[gamestate.weapon]+gamestate.weaponframe;
                 if (gamestate.ironsights == true)
                 {
                 shapenum = ironsightscale[gamestate.weapon];//+gamestate.weaponframe;
                 SimpleScaleShape((viewwidth/2)+(gamestate.bobber2/10),shapenum,viewheight+1+(gamestate.bobber/10));
                 } 
                 else 
                 {
                 shapenum = weaponscale[gamestate.weapon]+gamestate.weaponframe;
                 SimpleScaleShape(viewwidth - (viewwidth/3)-10+gamestate.bobber2,shapenum,viewheight+1+gamestate.bobber);
                 //SimpleScaleShape(viewwidth - (viewwidth/3),shapenum,viewheight+1);
                 }
        }    }
      } 
}

// Draw GasMask Gas Mask
 if (gamestate.mapon == 7)           //which map to draw gasmask on, counts from 0, ie first level is 0 second level 1 ect
 {
 SimpleScaleShape(viewwidth/(3.5),SPR_DEMO,viewheight);
 SimpleScaleShape(viewwidth-(viewwidth/3.5),SPR_DEATHCAM,viewheight);
//SimpleScaleShape(viewwidth-440,SPR_DEMO,viewheight);      //these were messed up if the player changed the aspect ratio
//SimpleScaleShape(viewwidth-200,SPR_DEATHCAM,viewheight);  //these were messed up if the player changed the aspect ratio
 }
 
 
//                     CA_CacheGrChunk (GOTGATLINGPIC);
//                     VWB_DrawPic (309, 150, GOTGATLINGPIC); // side side - up down
//                     UNCACHEGRCHUNK (GOTGATLINGPIC); 

 
    if (demorecord || demoplayback)
        SimpleScaleShape(viewwidth - (viewwidth/3),SPR_DEMO,viewheight+1);
}


//==========================================================================


/*
=====================
=
= CalcTics
=
=====================
*/

void CalcTics (void)
{
//
// calculate tics since last refresh for adaptive timing
//
    if (lasttimecount > (int32_t) GetTimeCount())
        lasttimecount = GetTimeCount();    // if the game was paused a LONG time

    uint32_t curtime = SDL_GetTicks();
    tics = (curtime * 7) / 100 - lasttimecount;
    if(!tics)
    {
        // wait until end of current tic
        SDL_Delay(((lasttimecount + 1) * 100) / 7 - curtime);
        tics = 1;
    }

    lasttimecount += tics;

    if (tics>MAXTICS)
        tics = MAXTICS;
}


//==========================================================================

void AsmRefresh()
{
    int32_t xstep,ystep;
    longword xpartial,ypartial;
    boolean playerInPushwallBackTile = tilemap[focaltx][focalty] == 64;

    for(pixx=0;pixx<viewwidth;pixx++)
    {
        short angl=midangle+pixelangle[pixx];
        if(angl<0) angl+=FINEANGLES;
        if(angl>=3600) angl-=FINEANGLES;
        if(angl<900)
        {
            xtilestep=1;
            ytilestep=-1;
            xstep=finetangent[900-1-angl];
            ystep=-finetangent[angl];
            xpartial=xpartialup;
            ypartial=ypartialdown;
        }
        else if(angl<1800)
        {
            xtilestep=-1;
            ytilestep=-1;
            xstep=-finetangent[angl-900];
            ystep=-finetangent[1800-1-angl];
            xpartial=xpartialdown;
            ypartial=ypartialdown;
        }
        else if(angl<2700)
        {
            xtilestep=-1;
            ytilestep=1;
            xstep=-finetangent[2700-1-angl];
            ystep=finetangent[angl-1800];
            xpartial=xpartialdown;
            ypartial=ypartialup;
        }
        else if(angl<3600)
        {
            xtilestep=1;
            ytilestep=1;
            xstep=finetangent[angl-2700];
            ystep=finetangent[3600-1-angl];
            xpartial=xpartialup;
            ypartial=ypartialup;
        }
        yintercept=FixedMul(ystep,xpartial)+viewy;
        xtile=focaltx+xtilestep;
        xspot=(word)((xtile<<mapshift)+((uint32_t)yintercept>>16));
        xintercept=FixedMul(xstep,ypartial)+viewx;
        ytile=focalty+ytilestep;
        yspot=(word)((((uint32_t)xintercept>>16)<<mapshift)+ytile);
        texdelta=0;

        // Special treatment when player is in back tile of pushwall
        if(playerInPushwallBackTile)
        {
            if(    pwalldir == di_east && xtilestep ==  1
                || pwalldir == di_west && xtilestep == -1)
            {
                int32_t yintbuf = yintercept - ((ystep * (64 - pwallpos)) >> 6);
                if((yintbuf >> 16) == focalty)   // ray hits pushwall back?
                {
                    if(pwalldir == di_east)
                        xintercept = (focaltx << TILESHIFT) + (pwallpos << 10);
                    else
                        xintercept = (focaltx << TILESHIFT) - TILEGLOBAL + ((64 - pwallpos) << 10);
                    yintercept = yintbuf;
                    tilehit = pwalltile;
                    HitVertWall();
                    continue;
                }
            }
            else if(pwalldir == di_south && ytilestep ==  1
                ||  pwalldir == di_north && ytilestep == -1)
            {
                int32_t xintbuf = xintercept - ((xstep * (64 - pwallpos)) >> 6);
                if((xintbuf >> 16) == focaltx)   // ray hits pushwall back?
                {
                    xintercept = xintbuf;
                    if(pwalldir == di_south)
                        yintercept = (focalty << TILESHIFT) + (pwallpos << 10);
                    else
                        yintercept = (focalty << TILESHIFT) - TILEGLOBAL + ((64 - pwallpos) << 10);
                    tilehit = pwalltile;
                    HitHorizWall();
                    continue;
                }
            }
        }

        do
        {
            if(ytilestep==-1 && (yintercept>>16)<=ytile) goto horizentry;
            if(ytilestep==1 && (yintercept>>16)>=ytile) goto horizentry;
vertentry:
            if((uint32_t)yintercept>mapheight*65536-1 || (word)xtile>=mapwidth)
            {
                if(xtile<0) xintercept=0;
                if(xtile>=mapwidth) xintercept=mapwidth<<TILESHIFT;
                if(yintercept<0) yintercept=0;
                if(yintercept>=(mapheight<<TILESHIFT)) yintercept=mapheight<<TILESHIFT;
                yspot=0xffff;
                tilehit=0;
                HitHorizBorder();
                break;
            }
            if(xspot>=maparea) break;
            tilehit=((byte *)tilemap)[xspot];
            if(tilehit)
            {
                if(tilehit&0x80)
                {
                    int32_t yintbuf=yintercept+(ystep>>1);
                    if((yintbuf>>16)!=(yintercept>>16))
                        goto passvert;
                    if((word)yintbuf<doorposition[tilehit&0x7f])
                        goto passvert;
                    yintercept=yintbuf;
                    xintercept=(xtile<<TILESHIFT)|0x8000;
                    HitVertDoor();
                }
                else
                {
                    if(tilehit==64)
                    {
                        if(pwalldir==di_west || pwalldir==di_east)
                        {
	                        int32_t yintbuf;
                            int pwallposnorm;
                            int pwallposinv;
                            if(pwalldir==di_west)
                            {
                                pwallposnorm = 64-pwallpos;
                                pwallposinv = pwallpos;
                            }
                            else
                            {
                                pwallposnorm = pwallpos;
                                pwallposinv = 64-pwallpos;
                            }
                            if(pwalldir == di_east && xtile==pwallx && ((uint32_t)yintercept>>16)==pwally
                                || pwalldir == di_west && !(xtile==pwallx && ((uint32_t)yintercept>>16)==pwally))
                            {
                                yintbuf=yintercept+((ystep*pwallposnorm)>>6);
                                if((yintbuf>>16)!=(yintercept>>16))
                                    goto passvert;

                                xintercept=(xtile<<TILESHIFT)+TILEGLOBAL-(pwallposinv<<10);
                                yintercept=yintbuf;
                                tilehit=pwalltile;
                                HitVertWall();
                            }
                            else
                            {
                                yintbuf=yintercept+((ystep*pwallposinv)>>6);
                                if((yintbuf>>16)!=(yintercept>>16))
                                    goto passvert;

                                xintercept=(xtile<<TILESHIFT)-(pwallposinv<<10);
                                yintercept=yintbuf;
                                tilehit=pwalltile;
                                HitVertWall();
                            }
                        }
                        else
                        {
                            int pwallposi = pwallpos;
                            if(pwalldir==di_north) pwallposi = 64-pwallpos;
                            if(pwalldir==di_south && (word)yintercept<(pwallposi<<10)
                                || pwalldir==di_north && (word)yintercept>(pwallposi<<10))
                            {
                                if(((uint32_t)yintercept>>16)==pwally && xtile==pwallx)
                                {
                                    if(pwalldir==di_south && (int32_t)((word)yintercept)+ystep<(pwallposi<<10)
                                            || pwalldir==di_north && (int32_t)((word)yintercept)+ystep>(pwallposi<<10))
                                        goto passvert;

                                    if(pwalldir==di_south)
                                        yintercept=(yintercept&0xffff0000)+(pwallposi<<10);
                                    else
                                        yintercept=(yintercept&0xffff0000)-TILEGLOBAL+(pwallposi<<10);
                                    xintercept=xintercept-((xstep*(64-pwallpos))>>6);
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                                else
                                {
                                    texdelta = -(pwallposi<<10);
                                    xintercept=xtile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                            }
                            else
                            {
                                if(((uint32_t)yintercept>>16)==pwally && xtile==pwallx)
                                {
                                    texdelta = -(pwallposi<<10);
                                    xintercept=xtile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                                else
                                {
                                    if(pwalldir==di_south && (int32_t)((word)yintercept)+ystep>(pwallposi<<10)
                                            || pwalldir==di_north && (int32_t)((word)yintercept)+ystep<(pwallposi<<10))
                                        goto passvert;

                                    if(pwalldir==di_south)
                                        yintercept=(yintercept&0xffff0000)-((64-pwallpos)<<10);
                                    else
                                        yintercept=(yintercept&0xffff0000)+((64-pwallpos)<<10);
                                    xintercept=xintercept-((xstep*pwallpos)>>6);
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                            }
                        }
                    }
                    else
                    {
                        xintercept=xtile<<TILESHIFT;
                        HitVertWall();
                    }
                }
                break;
            }
passvert:
            *((byte *)spotvis+xspot)=1;
            xtile+=xtilestep;
            yintercept+=ystep;
            xspot=(word)((xtile<<mapshift)+((uint32_t)yintercept>>16));
        }
        while(1);
        continue;

        do
        {
            if(xtilestep==-1 && (xintercept>>16)<=xtile) goto vertentry;
            if(xtilestep==1 && (xintercept>>16)>=xtile) goto vertentry;
horizentry:
            if((uint32_t)xintercept>mapwidth*65536-1 || (word)ytile>=mapheight)
            {
                if(ytile<0) yintercept=0;
                if(ytile>=mapheight) yintercept=mapheight<<TILESHIFT;
                if(xintercept<0) xintercept=0;
                if(xintercept>=(mapwidth<<TILESHIFT)) xintercept=mapwidth<<TILESHIFT;
                xspot=0xffff;
                tilehit=0;
                HitVertBorder();
                break;
            }
            if(yspot>=maparea) break;
            tilehit=((byte *)tilemap)[yspot];
            if(tilehit)
            {
                if(tilehit&0x80)
                {
                    int32_t xintbuf=xintercept+(xstep>>1);
                    if((xintbuf>>16)!=(xintercept>>16))
                        goto passhoriz;
                    if((word)xintbuf<doorposition[tilehit&0x7f])
                        goto passhoriz;
                    xintercept=xintbuf;
                    yintercept=(ytile<<TILESHIFT)+0x8000;
                    HitHorizDoor();
                }
                else
                {
                    if(tilehit==64)
                    {
                        if(pwalldir==di_north || pwalldir==di_south)
                        {
                            int32_t xintbuf;
                            int pwallposnorm;
                            int pwallposinv;
                            if(pwalldir==di_north)
                            {
                                pwallposnorm = 64-pwallpos;
                                pwallposinv = pwallpos;
                            }
                            else
                            {
                                pwallposnorm = pwallpos;
                                pwallposinv = 64-pwallpos;
                            }
                            if(pwalldir == di_south && ytile==pwally && ((uint32_t)xintercept>>16)==pwallx
                                || pwalldir == di_north && !(ytile==pwally && ((uint32_t)xintercept>>16)==pwallx))
                            {
                                xintbuf=xintercept+((xstep*pwallposnorm)>>6);
                                if((xintbuf>>16)!=(xintercept>>16))
                                    goto passhoriz;

                                yintercept=(ytile<<TILESHIFT)+TILEGLOBAL-(pwallposinv<<10);
                                xintercept=xintbuf;
                                tilehit=pwalltile;
                                HitHorizWall();
                            }
                            else
                            {
                                xintbuf=xintercept+((xstep*pwallposinv)>>6);
                                if((xintbuf>>16)!=(xintercept>>16))
                                    goto passhoriz;

                                yintercept=(ytile<<TILESHIFT)-(pwallposinv<<10);
                                xintercept=xintbuf;
                                tilehit=pwalltile;
                                HitHorizWall();
                            }
                        }
                        else
                        {
                            int pwallposi = pwallpos;
                            if(pwalldir==di_west) pwallposi = 64-pwallpos;
                            if(pwalldir==di_east && (word)xintercept<(pwallposi<<10)
                                    || pwalldir==di_west && (word)xintercept>(pwallposi<<10))
                            {
                                if(((uint32_t)xintercept>>16)==pwallx && ytile==pwally)
                                {
                                    if(pwalldir==di_east && (int32_t)((word)xintercept)+xstep<(pwallposi<<10)
                                            || pwalldir==di_west && (int32_t)((word)xintercept)+xstep>(pwallposi<<10))
                                        goto passhoriz;

                                    if(pwalldir==di_east)
                                        xintercept=(xintercept&0xffff0000)+(pwallposi<<10);
                                    else
                                        xintercept=(xintercept&0xffff0000)-TILEGLOBAL+(pwallposi<<10);
                                    yintercept=yintercept-((ystep*(64-pwallpos))>>6);
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                                else
                                {
                                    texdelta = -(pwallposi<<10);
                                    yintercept=ytile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                            }
                            else
                            {
                                if(((uint32_t)xintercept>>16)==pwallx && ytile==pwally)
                                {
                                    texdelta = -(pwallposi<<10);
                                    yintercept=ytile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                                else
                                {
                                    if(pwalldir==di_east && (int32_t)((word)xintercept)+xstep>(pwallposi<<10)
                                            || pwalldir==di_west && (int32_t)((word)xintercept)+xstep<(pwallposi<<10))
                                        goto passhoriz;

                                    if(pwalldir==di_east)
                                        xintercept=(xintercept&0xffff0000)-((64-pwallpos)<<10);
                                    else
                                        xintercept=(xintercept&0xffff0000)+((64-pwallpos)<<10);
                                    yintercept=yintercept-((ystep*pwallpos)>>6);
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                            }
                        }
                    }
                    else
                    {
                        yintercept=ytile<<TILESHIFT;
                        HitHorizWall();
                    }
                }
                break;
            }
passhoriz:
            *((byte *)spotvis+yspot)=1;
            ytile+=ytilestep;
            xintercept+=xstep;
            yspot=(word)((((uint32_t)xintercept>>16)<<mapshift)+ytile);
        }
        while(1);
    }
}

/*
====================
=
= WallRefresh
=
====================
*/

void WallRefresh (void)
{
    xpartialdown = viewx&(TILEGLOBAL-1);
    xpartialup = TILEGLOBAL-xpartialdown;
    ypartialdown = viewy&(TILEGLOBAL-1);
    ypartialup = TILEGLOBAL-ypartialdown;

    min_wallheight = viewheight;
    lastside = -1;                  // the first pixel is on a new wall
    AsmRefresh ();
    ScalePost ();                   // no more optimization on last post
}

void CalcViewVariables()
{
    viewangle = GetShockedAngle();  //was this before (changed for Zoom/Quake stuff): viewangle = player->angle;
    midangle = viewangle*(FINEANGLES/ANGLES);
    viewsin = sintable[viewangle];
    viewcos = costable[viewangle];
    if ( focallength < FOCALTHRESHOLD ) {
      viewx = player->x - FixedMul( focallength, viewcos );
      viewy = player->y + FixedMul( focallength, viewsin );
      } else {
      viewx = player->x - FixedMul( FOCALTHRESHOLD, viewcos );
      viewy = player->y + FixedMul( FOCALTHRESHOLD, viewsin );
      }
    //viewx = player->x - FixedMul(focallength,viewcos);    // the above if else statement replaces these two lines
    //viewy = player->y + FixedMul(focallength,viewsin);    // again for Zoom/Quake stuff

    focaltx = (short)(viewx>>TILESHIFT);
    focalty = (short)(viewy>>TILESHIFT);

    viewtx = (short)(player->x >> TILESHIFT);
    viewty = (short)(player->y >> TILESHIFT);
}

//==========================================================================

/*
========================
=
= ThreeDRefresh
=
========================
*/

void    ThreeDRefresh (void)
{
//
// clear out the traced array
//
    memset(spotvis,0,maparea);
    spotvis[player->tilex][player->tiley] = 1;       // Detect all sprites over player fix

    vbuf = VL_LockSurface(screenBuffer);
    vbuf+=screenofs;
    vbufPitch = bufferPitch;

    CalcViewVariables();

//
// follow the walls from there to the right, drawing as we go
//
    VGAClearScreen ();
#if defined(USE_FEATUREFLAGS) && defined(USE_STARSKY)
    if(GetFeatureFlags() & FF_STARSKY)
        DrawStarSky(vbuf, vbufPitch);
#endif

    WallRefresh ();

#if defined(USE_FEATUREFLAGS) && defined(USE_PARALLAX)
    if(GetFeatureFlags() & FF_PARALLAXSKY)
        DrawParallax(vbuf, vbufPitch);
#endif
#if defined(USE_FEATUREFLAGS) && defined(USE_CLOUDSKY)
    if(GetFeatureFlags() & FF_CLOUDSKY)
        DrawClouds(vbuf, vbufPitch, min_wallheight);
#endif
#ifdef USE_FLOORCEILINGTEX
    DrawFloorAndCeiling(vbuf, vbufPitch, min_wallheight);
#endif

//
// draw all the scaled images
//
    DrawScaleds();                  // draw scaled stuff
    if (messagetime >0)
       DrawMessage ();
    if (messagetime1 >0)
       DrawAwardMACHINEGUN ();
    if (messagetime2 >0)
       DrawAwardPISTOL ();
       DrawHltMessage ();
 
 
 
       
#if defined(USE_FEATUREFLAGS) && defined(USE_RAIN)
    if(GetFeatureFlags() & FF_RAIN)
        DrawRain(vbuf, vbufPitch);
#endif
#if defined(USE_FEATUREFLAGS) && defined(USE_SNOW)
    if(GetFeatureFlags() & FF_SNOW)
        DrawSnow(vbuf, vbufPitch);
#endif

    DrawPlayerWeapon ();    // draw player's hands and gasmask if any
    DrawKeys ();
    DrawRunBar ();
    if (gamestate.ironsights == false)
    DrawCrossHair ();       // draw player crosshairs
    if (gamestate.maphud == true)
    DrawLevelMap ();
    DrawHltMessage ();      // draw the health bar over the players weapon
if (gamestate.medical>0)
    DrawFirstAid ();      // draw number of health kits
if (gamestate.episode != 0)
    DrawKills ();
if (gamestate.episode == 0)

    gamestate.random = (rand() % 100 + 1);

                    // remeber there are three that must be changed for the lamp
if (gamestate.mapon == 5) //which map to draw a lamp on
{
  DrawLampMessage ();
}                    
else
{
   switch (gamestate.weapon)
   {
      case wp_knife:
//        if (gamestate.exseqtrigger == true) ExplosionSequence ();//bad fix
        if (gamestate.mapon == 7) // which map to have countdown on
        {
        CountDownEscapeTime ();
        if (gamestate.health >= 0)
           DrawEscapeTime ();
        if (gamestate.escapetime == 0)
           TakeDamage (1000,NULL);
        }
        DrawKnifeMessage ();    
         break;
      case wp_pistol:
//        if (gamestate.exseqtrigger == true) ExplosionSequence ();//bad fix
        if (gamestate.mapon == 7) // which map to have countdown on
        {
        CountDownEscapeTime ();
        if (gamestate.health >= 0)
           DrawEscapeTime ();
        if (gamestate.escapetime == 0)
           TakeDamage (1000,NULL);
        }
       DrawAmmo9mmMessage ();
       DrawClip1Message ();
         break;
      case wp_machinegun:
//        if (gamestate.exseqtrigger == true) ExplosionSequence ();//bad fix
        if (gamestate.mapon == 7) // which map to have countdown on
        {
        CountDownEscapeTime ();
        if (gamestate.health >= 0)
           DrawEscapeTime ();
        if (gamestate.escapetime == 0)
           TakeDamage (1000,NULL);
        }
       DrawAmmo30calMessage ();
       DrawClip2Message ();
          break;
     case wp_chaingun:
//        if (gamestate.exseqtrigger == true) ExplosionSequence ();//bad fix
        if (gamestate.mapon == 7) // which map to have countdown on
        {
        CountDownEscapeTime ();
        if (gamestate.health >= 0)
           DrawEscapeTime ();
        if (gamestate.escapetime == 0)
           TakeDamage (1000,NULL);
        }
       DrawAmmo30calMessage ();
       DrawClip3Message ();
         break;
     case wp_emplacedgun:
//        if (gamestate.exseqtrigger == true) ExplosionSequence ();//bad fix
        if (gamestate.mapon == 7) // which map to have countdown on
        {
        CountDownEscapeTime ();
        if (gamestate.health >= 0)
           DrawEscapeTime ();
        if (gamestate.escapetime == 0)
           TakeDamage (1000,NULL);
        }
       DrawEmplacedGunMessage ();
         break;
   }
#ifdef AUTOCOMIC
if (gamestate.endcomic == false)
{
if (gamestate.episode == 0)
{
    CountUpComicTime ();
    ComicCutScene ();
}
}
#endif
}


    //if(Keyboard[sc_Tab] && viewsize == 21 && gamestate.weapon != -1)
        //ShowActStatus();

    VL_UnlockSurface(screenBuffer);
    vbuf = NULL;

//
// show screen and time last cycle
//

    if (fizzlein)
    {
        FizzleFade(screenBuffer, screen, 0, 0,
            screenWidth, screenHeight, 20, false);
        fizzlein = false;

        lasttimecount = GetTimeCount();          // don't make a big tic count
    }
    else
    {
#ifndef REMDEBUG
        if (fpscounter)
        {
            fontnumber = 0;
            SETFONTCOLOR(7,127);
            PrintX=4; PrintY=1;
            VWB_Bar(0,0,50,10,bordercol);
            US_PrintSigned(fps);
            US_Print(" fps");
        }
#endif
        SDL_BlitSurface(screenBuffer, NULL, screen, NULL);
        SDL_UpdateRect(screen, 0, 0, 0, 0);
    }

#ifndef REMDEBUG
    if (fpscounter)
    {
        fps_frames++;
        fps_time+=tics;

        if(fps_time>35)
        {
            fps_time-=35;
            fps=fps_frames<<1;
            fps_frames=0;
        }
    }
#endif
}

/*
========================
=
= CountDownEscapeTime
=
= decrements escapetime using TimeCount
=
========================
*/

void CountDownEscapeTime (void)
{     
     if (gamestate.TimeCount%2)// == 0)
        gamestate.escapetime = (gamestate.escapetime-1);
     else
        gamestate.escapetime = gamestate.escapetime;     
}
#ifdef AUTOCOMIC
/*
========================
=
= CountDownEscapeTime (CountUpComicTime)
=
= increments escapetime (pagetime) using TimeCount (pagecount)
=
========================
*/

void CountUpComicTime (void)
{     
     if (gamestate.TimeCount%2)// == 0)
        gamestate.pagetime = (gamestate.pagetime+1);
     else
        gamestate.pagetime = gamestate.pagetime;     
}

#endif
/*
==================
=
= ExplosionSequence //bad fix
=
==================
*/

void ExplosionSequence ()
{
//basic time passage mechanic do not change
     if (gamestate.TimeCount%2)
        gamestate.exseqtime = (gamestate.exseqtime+1);
     else
        gamestate.exseqtime = gamestate.exseqtime;     

        // one trigger type with two events and stop/reset 
     if (gamestate.exseqtime == 25 ) SD_PlaySound (NEINSOVASSND);
     if (gamestate.exseqtime == 35 ) SD_PlaySound (SPIONSND);        //incoming warning
     if (gamestate.exseqtime == 60 ) StartQuakeEffect( 50 );
     if (gamestate.exseqtime == 90 ) SD_PlaySound (HITLERHASND);
     if (gamestate.exseqtime == 95 ) SD_PlaySound (MECHSTEPSND);
     if (gamestate.exseqtime == 130 ) SD_PlaySound (YEAHSND);
     if (gamestate.exseqtime == 140 ) StartQuakeEffect( 100 );
     if (gamestate.exseqtime == 220 ) StartQuakeEffect( 40 );
     if (gamestate.exseqtime == 270 ) SD_PlaySound (DONNERSND);//scared shitless
     if (gamestate.exseqtime == 280 ) StartQuakeEffect( 100 );
     if (gamestate.exseqtime == 285 ) SD_PlaySound (DEATHSCREAM2SND);//scream of pain
     if (gamestate.exseqtime == 290 ) SD_PlaySound (SCHEISTSND);
     if (gamestate.exseqtime == 300 ) StartQuakeEffect( 80 );
     if (gamestate.exseqtime == 310 ) SD_PlaySound (YEAHSND);
     if (gamestate.exseqtime == 410 ) StartQuakeEffect( 15 );
     if (gamestate.exseqtime == 480 ) SD_PlaySound (EINESND);
     if (gamestate.exseqtime == 500)
     {
      gamestate.exseqtrigger = false;
      gamestate.exseqtime = 0;
     }// end of trigger event sequence
}


/*
========================
=
= GetMessage
=
= gets ingame messages
=
========================
*/

void GetMessage (char *lastmessage)
{
   messagetime = 300; // time for message to display

   strcpy(gamestate.message, lastmessage);
}   



/*
========================
=
= DrawMessage
=
= displays ingame messages
=
========================
*/

void DrawMessage (void)
{
   messagetime-=tics;

   fontnumber = 0;

// message's shadow
   SETFONTCOLOR(0x0,0x1); // set the color
   PrintX=7; PrintY=1;
   US_Print(gamestate.message);

   SETFONTCOLOR(0x4,0x0); // set the color
   PrintX=8; PrintY=2; // position the message
   US_Print(gamestate.message); // print message
    DrawPlayBorderSides();
}

void DrawAwardMACHINEGUN (void)
{
   //messagetime = 300;
   messagetime1-=tics;
   fontnumber = 1;
   SETFONTCOLOR(0x0,0x1);
   PrintX=106; PrintY=23;
   US_Print(gamestate.message);
   SETFONTCOLOR(0x4,0x0);
   PrintX=107; PrintY=24;
   US_Print(gamestate.message);
}

void DrawAwardPISTOL (void)
{
   //messagetime = 300;
   messagetime2-=tics;
   fontnumber = 1;
   SETFONTCOLOR(0x0,0x1);
   PrintX=110; PrintY=23;
   US_Print(gamestate.message);
   US_PrintUnsigned(gamestate.random);
   SETFONTCOLOR(0x4,0x0);
   PrintX=111; PrintY=24;
   US_Print(gamestate.message);
   US_PrintUnsigned(gamestate.random);
}

void DrawKills (void)
{
//int random;
   fontnumber = 1;
   SETFONTCOLOR(0x0,0x1);
   PrintX=8; PrintY=12;
   US_Print("Kills: ");
   US_PrintUnsigned(gamestate.killed);
   SETFONTCOLOR(0x4,0x0);
   PrintX=9; PrintY=13;
   US_Print("Kills: ");
   US_PrintUnsigned(gamestate.killed);
}

void DrawRunBar (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=10; PrintY=10;
//   US_Print("");
//   US_PrintUnsigned (gamestate.runbar);
   SETFONTCOLOR(0x4,0x0);
   PrintX=11; PrintY=11;
//   US_Print("");
//   US_PrintUnsigned (gamestate.runbar);
   DrawPlayBorderSides ();

                    if (gamestate.runbar > 90 && gamestate.runbar < 150)
                    {
                     CA_CacheGrChunk (GOTGATLINGPIC);
                     VWB_DrawPic (309, 150, GOTGATLINGPIC); // side side - up down
                     UNCACHEGRCHUNK (GOTGATLINGPIC); 
                    }
                    if (gamestate.runbar < 93)
                    {
                     CA_CacheGrChunk (MUTANTBJPIC);
                     VWB_DrawPic (309, 150, MUTANTBJPIC); // side side - up down
                     UNCACHEGRCHUNK (MUTANTBJPIC); 
                    }
                    if (gamestate.runbar < 88)
                    {
                     CA_CacheGrChunk (N_9PIC);
                     VWB_DrawPic (309, 150, N_9PIC);
                     UNCACHEGRCHUNK (N_9PIC);                      
                    }
                    if (gamestate.runbar < 82)
                    {
                     CA_CacheGrChunk (N_8PIC);
                     VWB_DrawPic (309, 150, N_8PIC);
                     UNCACHEGRCHUNK (N_8PIC); 
                    }
                    if (gamestate.runbar < 77)
                    {
                     CA_CacheGrChunk (N_7PIC);
                     VWB_DrawPic (309, 150, N_7PIC);
                     UNCACHEGRCHUNK (N_7PIC);                      
                    }
                    if (gamestate.runbar < 71)
                    {
                     CA_CacheGrChunk (N_6PIC);
                     VWB_DrawPic (309, 150, N_6PIC);
                     UNCACHEGRCHUNK (N_6PIC); 
                    }
                    if (gamestate.runbar < 66)
                    {
                     CA_CacheGrChunk (N_5PIC);
                     VWB_DrawPic (309, 150, N_5PIC);
                     UNCACHEGRCHUNK (N_5PIC);                      
                    }
                    if (gamestate.runbar < 60)
                    {
                     CA_CacheGrChunk (N_4PIC);
                     VWB_DrawPic (309, 150, N_4PIC);
                     UNCACHEGRCHUNK (N_4PIC); 
                    }
                    if (gamestate.runbar < 55)
                    {
                     CA_CacheGrChunk (N_3PIC);
                     VWB_DrawPic (309, 150, N_3PIC);
                     UNCACHEGRCHUNK (N_3PIC);                      
                    }
                    if (gamestate.runbar < 49)
                    {
                     CA_CacheGrChunk (N_2PIC);
                     VWB_DrawPic (309, 150, N_2PIC);
                     UNCACHEGRCHUNK (N_2PIC); 
                    }
                    if (gamestate.runbar < 44)
                    {
                     CA_CacheGrChunk (N_1PIC);
                     VWB_DrawPic (309, 150, N_1PIC);
                     UNCACHEGRCHUNK (N_1PIC); 
                    }
                    if (gamestate.runbar < 38)
                    {
                     CA_CacheGrChunk (N_0PIC);
                     VWB_DrawPic (309, 150, N_0PIC);
                     UNCACHEGRCHUNK (N_0PIC); 
                    }
                    if (gamestate.runbar < 33)
                    {
                     CA_CacheGrChunk (N_BLANKPIC);
                     VWB_DrawPic (309, 150, N_BLANKPIC);
                     UNCACHEGRCHUNK (N_BLANKPIC); 
                    }
                    if (gamestate.runbar < 27)
                    {
                     CA_CacheGrChunk (SILVERKEYPIC);
                     VWB_DrawPic (309, 150, SILVERKEYPIC);
                     UNCACHEGRCHUNK (SILVERKEYPIC); 
                    }
                    if (gamestate.runbar < 22)
                    {
                     CA_CacheGrChunk (NOKEYPIC);
                     VWB_DrawPic (309, 150, NOKEYPIC);
                     UNCACHEGRCHUNK (NOKEYPIC); 
                    }
                    if (gamestate.runbar < 16)
                    {
                     CA_CacheGrChunk (GATLINGGUNPIC);
                     VWB_DrawPic (309, 150, GATLINGGUNPIC);
                     UNCACHEGRCHUNK (GATLINGGUNPIC); 
                    }
                    if (gamestate.runbar < 11)
                    {
                     CA_CacheGrChunk (MACHINEGUNPIC);
                     VWB_DrawPic (309, 150, MACHINEGUNPIC);
                     UNCACHEGRCHUNK (MACHINEGUNPIC); 
                    }
                    if (gamestate.runbar < 6)
                    {
                     CA_CacheGrChunk (GUNPIC);
                     VWB_DrawPic (309, 150, GUNPIC);
                     UNCACHEGRCHUNK (GUNPIC); 
                    }
                      if (gamestate.runbar < 1)
                    {
                     CA_CacheGrChunk (KNIFEPIC);
                     VWB_DrawPic (309, 150, KNIFEPIC);
                     UNCACHEGRCHUNK (KNIFEPIC); 
                    }



// Darkone's FPS Counter code
/*
    fps_frames++;
    fps_time+=tics;

    if(fps_time>35)
    {
          fps_time-=35;
          fps=fps_frames<<1;
          fps_frames=0;
    }
    SETFONTCOLOR(7,127);
    PrintX=8; PrintY=190;
    VWB_Bar(2,189,50,10,127);
    US_PrintSigned(fps);
    US_Print(" fps");
*/
//  End of FPS Counter
}



void DrawCrossHair (void)
{
     //For Question "B" - Search the routine... I forget the exact definitions,
     //but I think it's something like : X location - Y Start - Y End - Colour
     //to use <- for Hlin and of course the opposite for Vlin.... 
      VWB_Vlin ( 97, 102, 160, 165);
      VWB_Vlin ( 97, 102, 161, 165);
      VWB_Hlin (163, 168, 104, 165);
      VWB_Hlin (153, 158, 104, 165);
      VWB_Vlin (106, 111, 160, 165);
      VWB_Vlin (106, 111, 161, 165);
}


//=========================================================
//=
//=  DrawsLevelMap - tied to MapHud
//=  draws a simple overhead map of the world, which is updated
//=
//=========================================================
void DrawLevelMap (void)//strange code added http://diehardwolfers.areyep.com/viewtopic.php?t=1473
{
   int x=player->tilex,y=player->tiley;
   //int offx,offy,midx=249,midy=165,lenx=61,leny=31;
   int offx,offy,midx=5,midy=5,lenx=61,leny=31;
   offx = x*4/64; offy = y*34/64;

   for (x=0;x<lenx;x++)
      for (y=0;y<leny;y++)
      {
         if (tilemap[offx+x][offy+y] == 0)
         {
            VWB_Plot(midx+x,midy+y,0);
            if (actorat[offx+x][offy+y] != 0)
            {
               if ((unsigned)actorat[offx+x][offy+y] == 1)
                  VWB_Plot(midx+x,midy+y,158);
               else
                  VWB_Plot(midx+x,midy+y,150);
            }
         }
         else
            VWB_Plot(midx+x,midy+y,154);
      }

   VWB_Plot(player->tilex+midx-offx,player->tiley+midy-offy,140);
}
//=========================================================
//=========================================================
//=========================================================



void DrawHltMessage (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=8; PrintY=101;
   //US_Print("Health: ");
   //US_PrintUnsigned (gamestate.exseqtime);
   SETFONTCOLOR(0xe,0x0);
   PrintX=9; PrintY=100;
   //US_Print("Health: ");
   //US_PrintUnsigned (gamestate.health);
                     
                     CA_CacheGrChunk (FACE8APIC);
                     VWB_DrawPic (300, 184, FACE8APIC);
                     UNCACHEGRCHUNK (FACE8APIC); 


                    if (gamestate.health > 90)
                    {
                     CA_CacheGrChunk (FACE1APIC);
                     VWB_DrawPic (300, 150, FACE1APIC); // side side - up down
                     UNCACHEGRCHUNK (FACE1APIC); 
                    }
                    if (gamestate.health < 96)
                    {
                     CA_CacheGrChunk (FACE1BPIC);
                     VWB_DrawPic (300, 150, FACE1BPIC); // side side - up down
                     UNCACHEGRCHUNK (FACE1BPIC); 
                    }
                    if (gamestate.health < 90)
                    {
                     CA_CacheGrChunk (FACE1CPIC);
                     VWB_DrawPic (300, 150, FACE1CPIC);
                     UNCACHEGRCHUNK (FACE1CPIC);                      
                    }
                    if (gamestate.health < 85)
                    {
                     CA_CacheGrChunk (FACE2APIC);
                     VWB_DrawPic (300, 150, FACE2APIC);
                     UNCACHEGRCHUNK (FACE2APIC); 
                    }
                    if (gamestate.health < 80)
                    {
                     CA_CacheGrChunk (FACE2BPIC);
                     VWB_DrawPic (300, 150, FACE2BPIC);
                     UNCACHEGRCHUNK (FACE2BPIC);                      
                    }
                    if (gamestate.health < 75)
                    {
                     CA_CacheGrChunk (FACE2CPIC);
                     VWB_DrawPic (300, 150, FACE2CPIC);
                     UNCACHEGRCHUNK (FACE2CPIC); 
                    }
                    if (gamestate.health < 70)
                    {
                     CA_CacheGrChunk (FACE3APIC);
                     VWB_DrawPic (300, 150, FACE3APIC);
                     UNCACHEGRCHUNK (FACE3APIC);                      
                    }
                    if (gamestate.health < 65)
                    {
                     CA_CacheGrChunk (FACE3BPIC);
                     VWB_DrawPic (300, 150, FACE3BPIC);
                     UNCACHEGRCHUNK (FACE3BPIC); 
                    }
                    if (gamestate.health < 60)
                    {
                     CA_CacheGrChunk (FACE3CPIC);
                     VWB_DrawPic (300, 150, FACE3CPIC);
                     UNCACHEGRCHUNK (FACE3CPIC);                      
                    }
                    if (gamestate.health < 55)
                    {
                     CA_CacheGrChunk (FACE4APIC);
                     VWB_DrawPic (300, 150, FACE4APIC);
                     UNCACHEGRCHUNK (FACE4APIC); 
                    }
                    if (gamestate.health < 50)
                    {
                     CA_CacheGrChunk (FACE4BPIC);
                     VWB_DrawPic (300, 150, FACE4BPIC);
                     UNCACHEGRCHUNK (FACE4BPIC); 
                    }
                    if (gamestate.health < 45)
                    {
                     CA_CacheGrChunk (FACE4CPIC);
                     VWB_DrawPic (300, 150, FACE4CPIC);
                     UNCACHEGRCHUNK (FACE4CPIC); 
                    }
                    if (gamestate.health < 40)
                    {
                     CA_CacheGrChunk (FACE5APIC);
                     VWB_DrawPic (300, 150, FACE5APIC);
                     UNCACHEGRCHUNK (FACE5APIC); 
                    }
                    if (gamestate.health < 35)
                    {
                     CA_CacheGrChunk (FACE5BPIC);
                     VWB_DrawPic (300, 150, FACE5BPIC);
                     UNCACHEGRCHUNK (FACE5BPIC); 
                    }
                    if (gamestate.health < 30)
                    {
                     CA_CacheGrChunk (FACE5CPIC);
                     VWB_DrawPic (300, 150, FACE5CPIC);
                     UNCACHEGRCHUNK (FACE5CPIC); 
                    }
                    if (gamestate.health < 25)
                    {
                     CA_CacheGrChunk (FACE6APIC);
                     VWB_DrawPic (300, 150, FACE6APIC);
                     UNCACHEGRCHUNK (FACE6APIC); 
                    }
                    if (gamestate.health < 20)
                    {
                     CA_CacheGrChunk (FACE6BPIC);
                     VWB_DrawPic (300, 150, FACE6BPIC);
                     UNCACHEGRCHUNK (FACE6BPIC); 
                    }
                    if (gamestate.health < 15)
                    {
                     CA_CacheGrChunk (FACE6CPIC);
                     VWB_DrawPic (300, 150, FACE6CPIC);
                     UNCACHEGRCHUNK (FACE6CPIC); 
                    }
                    if (gamestate.health < 10)
                    {
                     CA_CacheGrChunk (FACE7APIC);
                     VWB_DrawPic (300, 150, FACE7APIC);
                     UNCACHEGRCHUNK (FACE7APIC); 
                    }
                    if (gamestate.health < 5)
                    {
                     CA_CacheGrChunk (FACE7BPIC);
                     VWB_DrawPic (300, 150, FACE7BPIC);
                     UNCACHEGRCHUNK (FACE7BPIC); 
                    }
                    if (gamestate.health < 1)
                    {
                     CA_CacheGrChunk (FACE7CPIC);
                     VWB_DrawPic (300, 150, FACE7CPIC);
                     UNCACHEGRCHUNK (FACE7CPIC); 
                    }
if  (gamestate.keys != 0) //drawkey if you have it     
{
    CA_CacheGrChunk (L_VPIC);
VWB_DrawPic (300, 135, L_VPIC);// 310 150
UNCACHEGRCHUNK (L_VPIC);
}
   DrawPlayBorderSides ();
} 

void DrawAmmo9mmMessage (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=30; PrintY=176;
   US_Print("l ");
   US_PrintUnsigned (gamestate.ammo[ammo_9mm]);
   SETFONTCOLOR(0x4,0x0);
   PrintX=31; PrintY=177;
   US_Print("l ");
   US_PrintUnsigned (gamestate.ammo[ammo_9mm]);
   DrawPlayBorderSides ();
}

void DrawAmmo30calMessage (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=30; PrintY=176;
   US_Print("l ");
   US_PrintUnsigned (gamestate.ammo[ammo_30cal]);
   SETFONTCOLOR(0x4,0x0);
   PrintX=31; PrintY=177;
   US_Print("l ");
   US_PrintUnsigned (gamestate.ammo[ammo_30cal]);
   DrawPlayBorderSides ();
}  

void DrawKnifeMessage (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=166;
   US_Print("Melee");
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=167;
   US_Print("Melee");

#ifdef AUTOCOMIC
    fontnumber = 0;
    SETFONTCOLOR(0x0,0x1);
    PrintX=5; PrintY=190;
    US_Print("cc");
    US_PrintUnsigned (gamestate.pagetime);
#endif

}

void DrawClip1Message (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=166;
   US_Print("Gew 98");
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=167;
   US_Print("Gew 98");

   fontnumber = 1;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=176;
   US_PrintUnsigned (gamestate.clip[1]);
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=177;
   US_PrintUnsigned (gamestate.clip[1]);
   DrawPlayBorderSides ();
}  

void DrawClip2Message (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=166;
   US_Print("Luger");
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=167;
   US_Print("Luger");

   fontnumber = 1;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=176;
   US_PrintUnsigned (gamestate.clip[2]);
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=177;
   US_PrintUnsigned (gamestate.clip[2]);
   DrawPlayBorderSides ();
}  

void DrawLampMessage (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=166;
   US_Print("Lamp");
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=167;
   US_Print("Lamp");
}

void DrawEmplacedGunMessage (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=166;
   US_Print("MG08");
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=167;
   US_Print("MG08");
}


void DrawPauseMessage (void)
{
   fontnumber = 1;
   SETFONTCOLOR(0x0,0x1);
   PrintX=107; PrintY=87;
   US_Print("Game Paused");
   SETFONTCOLOR(0x4,0x0);
   PrintX=108; PrintY=88;
   US_Print("Game Paused");
}

void DrawLoadingMessage (void)
{
   fontnumber = 1;
//   SETFONTCOLOR(0x0,0x1); //black
   SETFONTCOLOR(7,8); //grey almost white
   PrintX=125; PrintY=97;
   US_Print("Loading...");
   SETFONTCOLOR(0x4,0x0);
   PrintX=126; PrintY=98;
   US_Print("Loading...");
}

void DrawFirstAid (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=285; PrintY=5;
   //US_Print("Time Left: ");
   US_PrintUnsigned (gamestate.medical);
   SETFONTCOLOR(0x4,0x0);
   PrintX=286; PrintY=6;
   //US_Print("Time Left: ");
   US_PrintUnsigned (gamestate.medical);
                     CA_CacheGrChunk (PAUSEDPIC);
                     VWB_DrawPic (300, 5, PAUSEDPIC);
                     UNCACHEGRCHUNK (PAUSEDPIC); 
}



void DrawEscapeTime (void)
{
     int overalltime = gamestate.escapetime;
     int second = overalltime / 60;
     int minutes = second / 60;
     int fastseconds = gamestate.escapetime; 
     
     int seconds = second - (minutes * 60);
     int nanoseconds = seconds - (seconds * 60);
     
    fontnumber = 0;
    SETFONTCOLOR(0x0,0x1);
    PrintX=5; PrintY=5;
    US_Print("Time Left: 0");
    US_PrintUnsigned (minutes);
    SETFONTCOLOR(0x4,0x0);
    PrintX=6; PrintY=6;
    US_Print("Time Left: 0");
    US_PrintUnsigned (minutes);

    if (seconds < 10) {
       fontnumber = 0;
       SETFONTCOLOR(0x0,0x1);
       PrintX=90; PrintY=5;
       US_Print(": 0");
       US_PrintUnsigned (seconds);
       SETFONTCOLOR(0x4,0x0);
       PrintX=91; PrintY=6;
       US_Print(": 0");
       US_PrintUnsigned (seconds);
    }
    else {
         fontnumber = 0;
         SETFONTCOLOR(0x0,0x1);
         PrintX=90; PrintY=5;
         US_Print(": ");
         US_PrintUnsigned (seconds);
         SETFONTCOLOR(0x4,0x0);
         PrintX=91; PrintY=6;
         US_Print(": ");
         US_PrintUnsigned (seconds);         
    }

/*    fontnumber = 0;
    SETFONTCOLOR(0x0,0x1);
    PrintX=80; PrintY=5;
    US_Print(": ");
    US_PrintUnsigned (nanoseconds);
    SETFONTCOLOR(0x4,0x0);
    PrintX=81; PrintY=6;
    US_Print(": ");
    US_PrintUnsigned (nanoseconds);
*/}


void DrawClip3Message (void)
{
   fontnumber = 0;
   SETFONTCOLOR(0x0,0x1);
   PrintX=15; PrintY=166;
   US_Print("MP18");
   SETFONTCOLOR(0x4,0x0);
   PrintX=16; PrintY=167;
   US_Print("MP18");

   if(gamestate.clip[3] > 19)
   {
    fontnumber = 1;
    SETFONTCOLOR(0x0,0x1);
    PrintX=5; PrintY=176;
    US_PrintUnsigned (gamestate.clip[3]);
    SETFONTCOLOR(0x4,0x0);
    PrintX=6; PrintY=177;
    US_PrintUnsigned (gamestate.clip[3]);
    DrawPlayBorderSides ();
   }
   else
     {
       if (gamestate.clip[3] < 20)
       {
       fontnumber = 1;       
       if (gamestate.clip[3] < 10)
       {
       SETFONTCOLOR(0x0,0x1);
       PrintX=15; PrintY=176;
       US_PrintUnsigned (gamestate.clip[3]);
       SETFONTCOLOR(0x4,0x0);
       PrintX=16; PrintY=177;
       US_PrintUnsigned (gamestate.clip[3]);
       DrawPlayBorderSides ();
       }
       else
       {
       SETFONTCOLOR(0x0,0x1);
       PrintX=9; PrintY=176;
       US_PrintUnsigned (gamestate.clip[3]);
       SETFONTCOLOR(0x4,0x0);
       PrintX=10; PrintY=177;
       US_PrintUnsigned (gamestate.clip[3]);
       DrawPlayBorderSides ();
       }
     }
   } 
}






/*
=======================================
==
== Start of an attempt at either...
== A) A NPC conversation system.
== b) A log file system.
== c) A RPG battle system.
==
== Obviously never worked on it much.
==
=======================================
*/
void Battle (void)
{   
        ClearMemory ();
        CA_CacheGrChunk (STARTFONT + 0);
        ClearSplitVWB ();        
        MessageSmall ("Monster Attacks! \n \n Press Y to attack \n Press N to flee");
        //IN_ClearKeysDown ();
        //IN_Ack ();   
    
    do
    {
        IN_WaitAndProcessEvents();
    }
    while (!Keyboard[sc_Y] && !Keyboard[sc_N] && !Keyboard[sc_Escape]);

    if (Keyboard[sc_Y])
    {
    gamestate.health = gamestate.health + 50;
    }

    while (Keyboard[sc_Y] || Keyboard[sc_N] || Keyboard[sc_Escape])
        IN_WaitAndProcessEvents();

    IN_ClearKeysDown ();
    return;
}
/*
=======================================
==
= 
=
= 
==
=======================================
*/
#ifdef AUTOCOMIC
void ComicCutScene ()
{
     ////////////////////////////////////////////////////////////////////////////////
     if(gamestate.mapon == 0)//0 //intro cutscene
     {
      if ((gamestate.pagetime > -100) && (gamestate.pagetime < 50))
            {               
            CA_CacheGrChunk (H_RIGHTWINDOWPIC);
            VWB_DrawPic (0, 0, H_RIGHTWINDOWPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (H_RIGHTWINDOWPIC);
            }
      if ((gamestate.pagetime > 49) && (gamestate.pagetime < 100))
            {               
            CA_CacheGrChunk (C_DIGITITLEPIC);
            VWB_DrawPic (0, 0, C_DIGITITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_DIGITITLEPIC);
            }
      if ((gamestate.pagetime > 99) && (gamestate.pagetime < 150))
            {               
            CA_CacheGrChunk (C_MUSICTITLEPIC);
            VWB_DrawPic (0, 0, C_MUSICTITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_MUSICTITLEPIC); 
            }
      if ((gamestate.pagetime > 149) && (gamestate.pagetime < 200))
            {               
            CA_CacheGrChunk (L_WPIC);
            VWB_DrawPic (0, 0, L_WPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_WPIC); 
            }
      if ((gamestate.pagetime > 199) && (gamestate.pagetime < 250))
            {               
            CA_CacheGrChunk (L_XPIC);
            VWB_DrawPic (0, 0, L_XPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_XPIC); 
            }
      if ((gamestate.pagetime > 199) && (gamestate.pagetime < 250))
            {               
            CA_CacheGrChunk (L_YPIC);
            VWB_DrawPic (0, 0, L_YPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_YPIC); 
            }
      if ((gamestate.pagetime > 249) && (gamestate.pagetime < 300))
            {               
            CA_CacheGrChunk (L_ZPIC);
            VWB_DrawPic (0, 0, L_ZPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_ZPIC); 
            }
      if ((gamestate.pagetime > 299) && (gamestate.pagetime < 350))
            {               
            CA_CacheGrChunk (L_COLONPIC);
            VWB_DrawPic (0, 0, L_COLONPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_COLONPIC); 
            }
      if ((gamestate.pagetime > 349) && (gamestate.pagetime < 400))
            {               
            CA_CacheGrChunk (L_NUM0PIC);
            VWB_DrawPic (0, 0, L_NUM0PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM0PIC); 
            }
      if ((gamestate.pagetime > 399) && (gamestate.pagetime < 450))
            {               
            CA_CacheGrChunk (L_NUM1PIC);
            VWB_DrawPic (0, 0, L_NUM1PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM1PIC); 
            }
      if ((gamestate.pagetime > 449) && (gamestate.pagetime < 500))
            {               
            CA_CacheGrChunk (C_EPISODE6PIC);
            VWB_DrawPic (0, 0, C_EPISODE6PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_EPISODE6PIC); 
            }
      if ((gamestate.pagetime > 499) && (gamestate.pagetime < 550))
            {               
            CA_CacheGrChunk (L_GUYPIC);
            VWB_DrawPic (0, 0, L_GUYPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_GUYPIC); 
            }
      if ((gamestate.pagetime > 549) && (gamestate.pagetime < 600))
            {               
            CA_CacheGrChunk (C_CODEPIC);
            VWB_DrawPic (0, 0, C_CODEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_CODEPIC); 
            }
      if ((gamestate.pagetime > 599) && (gamestate.pagetime < 650))
            {               
            CA_CacheGrChunk (C_TIMECODEPIC);
            VWB_DrawPic (0, 0, C_TIMECODEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_TIMECODEPIC); 
            }
      if ((gamestate.pagetime > 649) && (gamestate.pagetime < 700))
            {               
            CA_CacheGrChunk (C_LEVELPIC);
            VWB_DrawPic (0, 0, C_LEVELPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_LEVELPIC); 
            }
      if ((gamestate.pagetime > 699) && (gamestate.pagetime < 750))
            {               
            CA_CacheGrChunk (C_NAMEPIC);
            VWB_DrawPic (0, 0, C_NAMEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_NAMEPIC); 
            }
      if ((gamestate.pagetime > 749) && (gamestate.pagetime < 800))
            {               
            CA_CacheGrChunk (C_SCOREPIC);
            VWB_DrawPic (0, 0, C_SCOREPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_SCOREPIC); 
            }
      if ((gamestate.pagetime > 799) && (gamestate.pagetime < 850))
            {               
            CA_CacheGrChunk (L_NUM2PIC);
            VWB_DrawPic (0, 0, L_NUM2PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM2PIC); 
            }
      if ((gamestate.pagetime > 849) && (gamestate.pagetime < 900))
            {               
            CA_CacheGrChunk (L_NUM3PIC);
            VWB_DrawPic (0, 0, L_NUM3PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM3PIC); 
            }
      if ((gamestate.pagetime > 899) && (gamestate.pagetime < 950))
            {               
            CA_CacheGrChunk (L_NUM4PIC);
            VWB_DrawPic (0, 0, L_NUM4PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM4PIC); 
            }
      if ((gamestate.pagetime > 949) && (gamestate.pagetime < 1000))
            {               
            CA_CacheGrChunk (L_NUM5PIC);
            VWB_DrawPic (0, 0, L_NUM5PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM5PIC); 
            }
      if ((gamestate.pagetime > 999) && (gamestate.pagetime < 1050))
            {               
            CA_CacheGrChunk (L_NUM6PIC);
            VWB_DrawPic (0, 0, L_NUM6PIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_NUM6PIC); 
            }
      if (gamestate.pagetime > 1051 )
            {               
            gamestate.endcomic = true;
            gamestate.pagetime = 0;
            }
      }
     ////////////////////////////////////////////////////////////////////////////////
     if(gamestate.mapon == 1)//medic cutscene
     {
      if ((gamestate.pagetime > -100) && (gamestate.pagetime < 70))
            {               
            CA_CacheGrChunk (L_DPIC);
            VWB_DrawPic (0, 0, L_DPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_DPIC);
            }
      if ((gamestate.pagetime > 69) && (gamestate.pagetime < 140))
            {               
            CA_CacheGrChunk (L_EPIC);
            VWB_DrawPic (0, 0, L_EPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_EPIC); 
            }
      if (gamestate.pagetime > 139)
            {               
            gamestate.endcomic = true;
            gamestate.pagetime = 0;
            }
      }
     ////////////////////////////////////////////////////////////////////////////////
     if(gamestate.mapon == 2) // flamethrower cutscene
     {
      if (gamestate.pagetime < 6)
            {               
            CA_CacheGrChunk (C_DIGITITLEPIC);
            VWB_DrawPic (0, 0, C_DIGITITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_DIGITITLEPIC);
            }
      if (gamestate.pagetime > 5 )
            {               
            CA_CacheGrChunk (C_MUSICTITLEPIC);
            VWB_DrawPic (0, 0, C_MUSICTITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_MUSICTITLEPIC); 
            }
      if (gamestate.pagetime > 10 )
            {               
            CA_CacheGrChunk (L_WPIC);
            VWB_DrawPic (0, 0, L_WPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_WPIC); 
            }
      if (gamestate.pagetime > 15 )
            {               
            CA_CacheGrChunk (L_XPIC);
            VWB_DrawPic (0, 0, L_XPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_XPIC); 
            }
      if (gamestate.pagetime > 20 )
            {               
            CA_CacheGrChunk (L_YPIC);
            VWB_DrawPic (0, 0, L_YPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_YPIC); 
            }
      if (gamestate.pagetime > 25 )
            {               
            CA_CacheGrChunk (L_ZPIC);
            VWB_DrawPic (0, 0, L_ZPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_ZPIC); 
            }
      if (gamestate.pagetime > 30 )
            {               
            gamestate.endcomic = true;
            }
      }
     ////////////////////////////////////////////////////////////////////////////////
     if(gamestate.mapon == 5)
     {
      if (gamestate.pagetime < 6)
            {               
            CA_CacheGrChunk (C_DIGITITLEPIC);
            VWB_DrawPic (0, 0, C_DIGITITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_DIGITITLEPIC);
            }
      if (gamestate.pagetime > 5 )
            {               
            CA_CacheGrChunk (C_MUSICTITLEPIC);
            VWB_DrawPic (0, 0, C_MUSICTITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_MUSICTITLEPIC); 
            }
      if (gamestate.pagetime > 10 )
            {               
            CA_CacheGrChunk (L_WPIC);
            VWB_DrawPic (0, 0, L_WPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_WPIC); 
            }
      if (gamestate.pagetime > 15 )
            {               
            CA_CacheGrChunk (L_XPIC);
            VWB_DrawPic (0, 0, L_XPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_XPIC); 
            }
      if (gamestate.pagetime > 20 )
            {               
            CA_CacheGrChunk (L_YPIC);
            VWB_DrawPic (0, 0, L_YPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_YPIC); 
            }
      if (gamestate.pagetime > 25 )
            {               
            CA_CacheGrChunk (L_ZPIC);
            VWB_DrawPic (0, 0, L_ZPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_ZPIC); 
            }
      if (gamestate.pagetime > 30 )
            {               
            gamestate.endcomic = true;
            }
      }
     ////////////////////////////////////////////////////////////////////////////////
     if(gamestate.mapon == 7)
     {
      if (gamestate.pagetime < 6)
            {               
            CA_CacheGrChunk (C_DIGITITLEPIC);
            VWB_DrawPic (0, 0, C_DIGITITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_DIGITITLEPIC);
            }
      if (gamestate.pagetime > 5 )
            {               
            CA_CacheGrChunk (C_MUSICTITLEPIC);
            VWB_DrawPic (0, 0, C_MUSICTITLEPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (C_MUSICTITLEPIC); 
            }
      if (gamestate.pagetime > 10 )
            {               
            CA_CacheGrChunk (L_WPIC);
            VWB_DrawPic (0, 0, L_WPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_WPIC); 
            }
      if (gamestate.pagetime > 15 )
            {               
            CA_CacheGrChunk (L_XPIC);
            VWB_DrawPic (0, 0, L_XPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_XPIC); 
            }
      if (gamestate.pagetime > 20 )
            {               
            CA_CacheGrChunk (L_YPIC);
            VWB_DrawPic (0, 0, L_YPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_YPIC); 
            }
      if (gamestate.pagetime > 25 )
            {               
            CA_CacheGrChunk (L_ZPIC);
            VWB_DrawPic (0, 0, L_ZPIC);                // side to side - top to bttom
            UNCACHEGRCHUNK (L_ZPIC); 
            }
      if (gamestate.pagetime > 30 )
            {               
            gamestate.endcomic = true;
            }
      }
}
#endif
